// PACKAGES USED
const { Command } = require('@oclif/core');
const inquirer = require('inquirer');
const fs = require('fs');
const path = require('path');
const oracledb = require('oracledb');
// UTILITIES
const { initLogger } = require(path.join(__dirname, '../../utils/logger.js'));
const logger = initLogger('config');
const { fileChecker, serviceFiles } = require("../../utils/fileChecker");

// CHECKING IF CONFIG FILE EXISTS AND REQUIRING THE DATA IN IT
let config, encrypt;
if (fs.existsSync(__dirname.slice(0, __dirname.indexOf('src')) + 'config.json')) config = require('../../../config.json');
let filePaths = {
    nodeconsumer: '/Handlers/config.json',
    nrestserver: '/fundsnservices/dbconfig.json',
    jobnodered: '/config.json',
    iwfetl: '/custom/config.json',
    appserver: '/standalone/configuration/standalone.xml',
    fixInterface2: '/FIXTrading/config/credFix.cfg',
    fixInterface4: '/FIXTrading/config/credFix.cfg',
    mailingEngine: '/bin/outbox_config_1.txt',
};

// CREATED AN OBJECT THAT HAS KEYS AS KEYWORD NAMES OF SERVICES AND A FUNCTION THAT MAKES CHANGES TO THE PARTICULAR SERVICE FILES
const processFunctions = {
    // FUNCTION TO MAKE CHANGES TO JBOSS standalone.xml SERVER FILES
    appserver: ({ username, password, connectionString }) => {
        let filePath = path.join(config.appserver, filePaths.appserver);
        if (fileChecker([filePath]))
            fs.readFile(filePath, { encoding: "utf-8", flag: "r" }, (err, data) => {
                if (err) {
                    `An error occurred while reading ${filePath}`;
                    logger.error(err.message);
                } else {
                    data = data.replace(
                        /<connection-url>(?:[^>]*)<\/connection-url>/g,
                        `<connection-url>jdbc:oracle:thin:@${connectionString}<\/connection-url>`
                    );
                    data = data.replace(
                        /<user-name>(?:[^>]*)<\/user-name>/g,
                        `<user-name>${username}<\/user-name>`
                    );
                    data = data.replace(
                        /<password>(?:[^>]*)<\/password>/g,
                        `<password>${password}<\/password>`
                    );
                    fs.writeFile(filePath, data, (err) => {
                        if (err) {
                            `An error occurred while writing changes in the ${filePath}.`;
                            logger.error(err.message);
                        }
                    });
                }
            });
        else logger.error("Appserver file does not exist. Run config:init");
    },
    // FUNCTION TO MAKE CHANGES IWFETL config.json FILES
    iwfetl: ({ username, password, connectionString, enDbPw }) => {
        let filePath = path.join(config.iwfetl, filePaths.iwfetl);
        try {
            if (fileChecker([filePath])) {
                const data = require(filePath);
                if (!data.database) data.database = {};
                if (!data.database.src) data.database.src = {};
                if (!data.database.dest) data.database.dest = {};
                data.database.src['user'] = username;
                data.database.src['password'] = enDbPw || password;
                data.database.src['connectString'] = connectionString;
                data.database.dest['user'] = username;
                data.database.dest['password'] = enDbPw || password;
                data.database.dest['connectString'] = connectionString;
                fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
                    if (err) {
                        logger.error(`An error occurred while writing changes in ${filePath}`);
                        logger.error(err.message);
                    }
                });
            } else logger.error('ETL file does not exist. Run config:init');
        } catch (e) {
            logger.error(`An error occurred while reading ${filePath}`);
            logger.error(e.message);
        }
    },
    // FUNCTION TO MAKE CHANGES TO JOBNODERED config.json FILE
    jobnodered: ({ username, password, connectionString, enDbPw }) => {
        let filePath = path.join(config.jobnodered, filePaths.jobnodered);
        try {
            if (fileChecker([filePath])) {
                const data = require(filePath);
                if (!data.databases) data.databases = {};
                if (!data.databases.ideal) data.databases.ideal = {};
                data.databases.ideal.user = username;
                data.databases.ideal.password = enDbPw || password;
                data.databases.ideal.connectstring = connectionString;
                fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
                    if (err) {
                        logger.error(`An error occurred while writing changes in ${filePath}`);
                        logger.error(err.message);
                    }
                });
            } else logger.error('job-center-node-red file does not exist. Run config:init');
        } catch (e) {
            logger.error(`An error occurred while reading ${filePath}`);
            logger.error(e.message);
        }
    },
    // FUNCTION TO MAKE CHANGES TO NRESTSERVER dbconfig.json FILE
    nrestserver: ({ username, password, connectionString, enDbPw }) => {
        let filePath = path.join(config.nrestserver, filePaths.nrestserver);
        try {
            if (fileChecker([filePath])) {
                const data = require(filePath);
                if (!data.oracledb)
                    data.oracledb = [
                        {
                            user: username,
                            password: enDbPw || password,
                            connectString: connectionString,
                        },
                    ];
                data.oracledb[0] = {
                    ...data.oracledb[0],
                    user: username,
                    password: enDbPw || password,
                    connectString: connectionString,
                };
                fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
                    if (err) {
                        logger.error(`An error occurred while writing changes in ${filePath}`);
                        logger.error(err.message);
                    }
                });
            } else logger.error('nrest-server file does not exist. Run config:init');
        } catch (e) {
            logger.error(`An error occurred while reading ${filePath}`);
            logger.error(e.message);
        }
    },
    // FUNCTION TO MAKE CHANGES NODECONSUMER config.json FILE
    nodeconsumer: ({ username, password, connectionString, enDbPw }) => {
        let filePath = path.join(config.nodeconsumer, filePaths.nodeconsumer);
        try {
            if (fileChecker([filePath])) {
                const data = require(filePath);
                if (!data.oracledb) data.oracledb = [];
                if (!data.oracledb[0]) data.oracledb[0] = {};
                data.oracledb[0]['username'] = username;
                data.oracledb[0]['password'] = enDbPw || password;
                data.oracledb[0]['connectString'] = connectionString;
                fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
                    if (err) {
                        logger.error(`An error occurred while writing changes in ${filePath}`);
                        logger.error(err.message);
                    }
                });
            } else logger.error('funds-node-consumer file does not exist. Run config:init');
        } catch (e) {
            logger.error(`An error occurred while reading ${filePath}`);
            logger.error(e.message);
        }
    },
    // FUNCTION TO MAKE CHANGES FIX INTERFACE 4.2 credFix.cgf FILE
    fixInterface2: ({ username, password, connectionString, enDbPw }) => {
        let filePath = path.join(config.fixInterface2, filePaths.fixInterface2);
        if (fileChecker([filePath])) {
            fs.readFile(filePath, { encoding: 'utf-8', flag: 'r' }, async (err, data) => {
                if (err) {
                    logger.error(`An error occurred while reading ${filePath}`);
                    logger.error(err.message);
                } else {
                    if (data.search(/JdbcURL=jdbc:oracle:thin:@(.*)/g) !== -1) data = data.replace(/JdbcURL=jdbc:oracle:thin:@(.*)/g, 'JdbcURL=jdbc:oracle:thin:@' + connectionString);
                    else data = data + '\nJdbcURL=jdbc:oracle:thin:@' + connectionString;
                    if (data.search(/JdbcUser(.*)/g) !== -1) data = data.replace(/JdbcUser(.*)/g, 'JdbcUser=' + username);
                    else data = data + '\nJdbcUser=' + username;
                    if (data.search(/JdbcPassword(.*)/g) !== -1) data = data.replace(/JdbcPassword(.*)/g, 'JdbcPassword=' + password);
                    else data = data + '\nJdbcPassword=' + password;
                    fs.writeFile(filePath, data, (err) => {
                        if (err) {
                            logger.error(`An error occurred while writing changes in ${filePath}`);
                            logger.error(err.message);
                        }
                    });
                }
            });
        } else logger.error('Fix Interface 4.2 file does not exist. Run config:init');
    },
    // FUNCTION TO MAKE CHANGES FIX INTERFACE 4.4 credFix.cgf FILE
    fixInterface4: ({ username, password, connectionString, enDbPw }) => {
        let filePath = path.join(config.fixInterface4, filePaths.fixInterface4);
        if (fileChecker([filePath])) {
            fs.readFile(filePath, { encoding: 'utf-8', flag: 'r' }, async (err, data) => {
                if (err) {
                    logger.error(`An error occurred while reading ${filePath}`);
                    logger.error(err.message);
                } else {
                    if (data.search(/JdbcURL=jdbc:oracle:thin:@(.*)/g) !== -1) data = data.replace(/JdbcURL=jdbc:oracle:thin:@(.*)/g, 'JdbcURL=jdbc:oracle:thin:@' + connectionString);
                    else data = data + '\nJdbcURL=jdbc:oracle:thin:@' + connectionString;
                    if (data.search(/JdbcUser(.*)/g) !== -1) data = data.replace(/JdbcUser(.*)/g, 'JdbcUser=' + username);
                    else data = data + '\nJdbcUser=' + username;
                    if (data.search(/JdbcPassword(.*)/g) !== -1) data = data.replace(/JdbcPassword(.*)/g, 'JdbcPassword=' + password);
                    else data = data + '\nJdbcPassword=' + password;
                    fs.writeFile(filePath, data, (err) => {
                        if (err) {
                            logger.error(`An error occurred while writing changes in ${filePath}`);
                            logger.error(err.message);
                        }
                    });
                }
            });
        } else logger.error('Fix Interface 4.4 file does not exist. Run config:init');
    },
    // FUNCTION TO MAKE CHANGES MAILING ENGINE outbox_config_1.txt FILE
    mailingEngine: ({ username, password, connectionString, enDbPw }) => {
        let filePath = path.join(config.mailingEngine, filePaths.mailingEngine);
        if (fileChecker([filePath])) {
            fs.readFile(filePath, { encoding: 'utf-8', flag: 'r' }, async (err, data) => {
                if (err) {
                    logger.error(`An error occurred while reading ${filePath}`);
                    logger.error(err.message);
                } else {
                    if (data.search(/dbName\s*?=\s*?jdbc:oracle:thin:@(.*)/g) !== -1) data = data.replace(/dbName\s*?=\s*?jdbc:oracle:thin:@(.*)/g, 'dbName=jdbc:oracle:thin:@' + connectionString);
                    else data = data + '\ndbName=jdbc:oracle:thin:@' + connectionString;
                    if (data.search(/username\s*?=\s*?(.*)/g) !== -1) data = data.replace(/username\s*?=\s*?(.*)/g, 'username=' + username);
                    else data = data + '\nusername=' + username;
                    if (data.search(/password\s*?=\s*?(.*)/g) !== -1) data = data.replace(/password\s*?=\s*?(.*)/g, 'password=' + password);
                    else data = data + '\npassword=' + password;
                    fs.writeFile(filePath, data, (err) => {
                        if (err) {
                            logger.error(`An error occurred while writing changes in ${filePath}`);
                            logger.error(err.message);
                        }
                    });
                }
            });
        } else logger.error('Fix Interface 4.4 file does not exist. Run config:init');
    },
};

// MAIN COMMAND FUNCTION
class DatabaseCommand extends Command {
    async run() {
        // CHECKING IF CONFIG FILES EXIST OR NOT
        if (config) {
            // CALLING FILE CHECKER FUNCTION WHICH CHECKS THAT ALL THE FILES BEING USED EXIST ARE READABLE AND WRITEABLE
            // THE SERVICE FILES FUNCTION GIVES BACK US AN ARRAY OF ABSOLUTE PATH FROM THE ROOT DIRECTORY FOR DIFFERENT SERVICE FILES
            if (fileChecker(serviceFiles(config, filePaths))) {
                let database,
                    connection,
                    flag = true;
                // DO WHILE LOOP TO CHECK IF DATABASE CREDENTIALS PROVIDED CONNECT US TO THE DATABASE
                do {
                    try {
                        // PROMPTING THE USER FOR DATABASE INPUTS
                        database = await inquirer.prompt([
                            {
                                message: 'Enter database username',
                                name: 'username',
                                validate: (res) => {
                                    if (res === '') return 'Enter a value';
                                    return true;
                                },
                            },
                            {
                                message: 'Enter database password',
                                name: 'password',
                                type: 'password',
                                validate: (res) => {
                                    if (res === '') return 'Enter a value';
                                    return true;
                                },
                            },
                            {
                                message: 'Enter database host',
                                name: 'host',
                                validate: (res) => {
                                    if (res === '') return 'Enter a value';
                                    return true;
                                },
                            },
                            {
                                message: 'Enter database port',
                                name: 'port',
                                validate: (res) => {
                                    if (res === '') return 'Enter a value';
                                    if (!/^\d+$/.test(res)) return 'Enter a number';
                                    return true;
                                },
                            },
                            {
                                message: 'Enter database SID',
                                name: 'sid',
                                validate: (res) => {
                                    if (res === '') return 'Enter a value';
                                    if (res.search(/^[a-zA-Z0-9_.]+$/) === -1) return 'No special characters allowed';
                                    return true;
                                },
                            },
                            {
                                message: 'Would you like to encrypt sensitive data?',
                                name: 'choice_encrypt',
                                type: 'list',
                                choices: ['Yes', 'No'],
                            },
                        ]);
                        // FORMING CONNECTION STRING KEY AND REMOVING THE SEPARATE KEYS OF HOST, PORT, SID
                        database.connectionString = database.host + ':' + database.port + '/' + database.sid;
                        flag = false;
                        connection = await oracledb.getConnection(database);
                    } catch (error) {
                        logger.error('Could not connect to database');
                        flag = true;
                    } finally {
                        if (connection) {
                            try {
                                await connection.close();
                            } catch (err) {
                                logger.error(err.message);
                            }
                        }
                    }
                } while (flag);

                database.enDbPw = null;
                if (database.choice_encrypt === 'Yes') {
                    // Encrypt function
                    encrypt = require('../../utils/encrypt')(path.join(__dirname, '../../../', 'keys/public.pem'));
                    database.enDbPw = encrypt(database.password);
                }

                // MAPPING OVER THE CONFIG OBJECT TO CALL CHANGES ON ONLY THOSE SERVICES THAT HAVE BEEN INITIALIZED AND THEIR PATHS ARE PRESENT IN config.json
                Object.keys(config).forEach((service) => {
                    if (processFunctions[service]) processFunctions[service](database);
                });
                logger.info('Database Initialisation Successful');
            }
        } else logger.error('Config file not found. Run config:init command');
    }
}

DatabaseCommand.description = `Initialize or update the database details. For any errors reinitialize using config:init command`;
module.exports = DatabaseCommand;
