// PACKAGES USED
const { Command } = require('@oclif/core');
const inquirer = require('inquirer');
const fs = require('fs');
const path = require('path');

// UTILITIES
const logger = require("../../utils/logger");
const { fileChecker } = require("../../utils/fileChecker");

// CHECKING IF CONFIG FILE EXISTS AND REQUIRING THE DATA IN IT
let config;
if (fs.existsSync(__dirname.slice(0, __dirname.indexOf("src")) + "config.json"))
  config = require("../../../config.json");

// CREATED AN OBJECT THAT HAS KEYS AS KEYWORD NAMES OF SERVICES AND A FUNCTION THAT MAKES CHANGES TO THE PARTICULAR SERVICE FILES
const processFunctions = {
  // FUNCTION TO EDIT THE CONFIG FILES IN APPSERVER
  appserver:
    // CHECKING IF PLATFORM IS WINDOWS OR LINUX
    process.platform == "win32"
      ? ({ java, min, max }) => {
          let filePath = path.join(
            config.appserver,
            "/bin/standalone.conf.bat"
          );
          if (fileChecker([filePath])) {
            fs.readFile(
              filePath,
              { encoding: "utf-8", flag: "r" },
              (err, data) => {
                if (err) {
                  logger.error(`An error occurred while reading ${filePath}`);
                  logger.error(err.message);
                } else {
                  if (
                    data.search(/set\s*?"JAVA_HOME\s*?=(.*)/g) !== -1 &&
                    data.slice(
                      data.search(/rem\s*?set\s*?"JAVA_HOME\s*?=(.*)/g),
                      data.search(/set\s*?"JAVA_HOME\s*?=(.*)/g)
                    ) !== "rem "
                  )
                    data = data.replace(
                      /set\s*?"JAVA_HOME=(.*)/,
                      `set "JAVA_HOME=${java}"`
                    );
                  else {
                    let splitIdx = data.search(
                      /rem\s*?set\s*?"JAVA_HOME=(.*)/g
                    );
                    data =
                      data.slice(0, splitIdx) +
                      `\nset "JAVA_HOME=${java}"\n` +
                      data.slice(splitIdx, data.length);
                  }
                  if (
                    data.search(
                      /JAVA_OPTS\s*?=\s*?-Xms(.*)M\s*?-Xmx(.*)M\s*?/g
                    ) !== -1
                  )
                    data = data.replace(
                      /JAVA_OPTS\s*?=\s*?-Xms(.*)M\s*?-Xmx(.*)M\s*?/g,
                      `JAVA_OPTS=-Xms${min}M -Xmx${max}M `
                    );
                  fs.writeFile(filePath, data, (err) => {
                    if (err) {
                      logger.error(
                        `An error occurred while writing changes in ${filePath}`
                      );
                      logger.error(err.message);
                    }
                  });
                }
              }
            );
          } else
            logger.error(`${filePath} file does not exist. Run config:init`);
        }
      : ({ java, min, max }) => {
          let filePath = path.join(config.appserver, "/bin/standalone.conf");
          if (fileChecker([filePath])) {
            fs.readFile(
              filePath,
              { encoding: "utf-8", flag: "r" },
              (err, data) => {
                if (err) {
                  logger.error(`An error occurred while reading ${filePath}`);
                  logger.error(err.message);
                } else {
                  if (
                    data.search(/JAVA_HOME\s*?=\s*?"(.*)/g) !== -1 &&
                    data.slice(
                      data.search(/#\s*?JAVA_HOME\s*?=\s*?"(.*)/g),
                      data.search(/\s*?JAVA_HOME\s*?=\s*?"(.*)/g)
                    ) !== "#"
                  )
                    data = data.replace(
                      /JAVA_HOME\s*?=\s*?"(.*)/,
                      `JAVA_HOME="${java}"`
                    );
                  else {
                    let splitIdx = data.search(/#\s*?JAVA_HOME\s*?=\s*?"(.*)/g);
                    data =
                      data.slice(0, splitIdx) +
                      `\JAVA_HOME="${java}"\n` +
                      data.slice(splitIdx, data.length);
                  }
                  if (
                    data.search(
                      /JAVA_OPTS\s*?=\s*?"\s*?-Xms(.*)m\s*?-Xmx(.*)m/g
                    ) !== -1
                  )
                    data = data.replace(
                      /JAVA_OPTS\s*?=\s*?"\s*?-Xms(.*)m\s*?-Xmx(.*)m/g,
                      `JAVA_OPTS="-Xms${min}m -Xmx${max}m `
                    );
                  fs.writeFile(filePath, data, (err) => {
                    if (err) {
                      logger.error(
                        `An error occurred while writing changes in ${filePath}`
                      );
                      logger.error(err.message);
                    }
                  });
                }
              }
            );
          } else
            logger.error(`${filePath} file does not exist. Run config:init`);
        },
  // FUNCTION TO EDIT THE CONFIG FILES IN MAIL ENGINE
  // mailingEngine: ({ java }) => {
  //   let filePath = [
  //     path.join(
  //       config.mailingEngine,
  //       `/OutboxMail_1.${process.platform === "win32" ? "bat" : "sh"}`
  //     ),
  //     path.join(
  //       config.mailingEngine,
  //       `/Scheduler_1.${process.platform === "win32" ? "bat" : "sh"}`
  //     ),
  //   ];
  //   if (fileChecker(filePath)) {
  //     filePath.forEach((path) => {
  //       fs.readFile(path, { encoding: "utf-8", flag: "r" }, (err, data) => {
  //         if (err) {
  //           logger.error(`An error occurred while reading ${path}`);
  //           logger.error(err.message);
  //         } else {
  //           if (data.search(/JAVA_HOME\s*?=(.*)/g) !== -1)
  //             data = data.replace(/JAVA_HOME\s*?=(.*)/, `JAVA_HOME=${java}`);
  //           else data = data + `\nJAVA_HOME=${java}`;
  //           fs.writeFile(path, data, (err) => {
  //             if (err) {
  //               logger.error(
  //                 `An error occurred while writing changes in ${path}`
  //               );
  //               logger.error(err.message);
  //             }
  //           });
  //         }
  //       });
  //     });
  //   } else logger.error(`${filePath} file does not exist. Run config:init`);
  // },
};

class JavaCommand extends Command {
  async run() {
    if (config) {
      // CHECKING FILES USED USING FILE CHECKER FUNCTION
      let filesUsed = [
        path.join(
          config.appserver,
          `/bin/standalone.conf${process.platform === "win32" ? ".bat" : ""}`
        )
        // path.join(config.mailingEngine, "/OutboxMail_1.bat"),
        // path.join(config.mailingEngine, "/Scheduler_1.bat"),
      ];
      if (fileChecker(filesUsed)) {
        // TAKING INPUTS FROM USERS
        let inputs = await inquirer.prompt([
          {
            message: "Enter java folder path",
            name: "java",
            validate: (res) => {
              if (res === "") return "Enter a value";
              if (!fs.existsSync(res)) return "Path not found";
              return true;
            },
          },
          {
            message: "Enter minimum RAM allocation (MB) for jboss/wildfly",
            name: "min",
            default: 64,
            validate: (res) => {
              if (res === "") return "Enter a value";
              if (!/^\d+$/.test(res)) return "Enter a number";
              return true;
            },
          },
          {
            message: "Enter maximum RAM allocation (MB) for jboss/wildfly",
            name: "max",
            default: 1024,
            validate: (res) => {
              if (res === "") return "Enter a value";
              if (!/^\d+$/.test(res)) return "Enter a number";
              return true;
            },
          },
        ]);
        processFunctions["appserver"](inputs);
        // processFunctions["mailingEngine"](inputs);
        logger.info("Java Initialisation Successful");
      }
    } else logger.error("Config file not found. Run config:init command");
  }
}

JavaCommand.description = `Update the java details in the JBOSS folder. For any errors reinitialize using config:init command`;
module.exports = JavaCommand;
