// PACKAGES USED
const { Command } = require('@oclif/core');
const inquirer = require('inquirer');
const fs = require('fs');
const path = require('path');
const { initLogger } = require(path.join(__dirname, '../../utils/logger.js'));
const logger = initLogger('config');

// UTILITIES
const { fileChecker, serviceFiles } = require("../../utils/fileChecker");

// CHECKING IF CONFIG FILE EXISTS AND REQUIRING THE DATA IN IT
let config, encrypt;
if (fs.existsSync(__dirname.slice(0, __dirname.indexOf("src")) + "config.json"))
  config = require("../../../config.json");
let filePaths = {
  iwfetl: "/custom/config.json",
  jobnodered: "/config.json",
  nrestserver: "/fundsnservices/dbconfig.json",
  nodeconsumer: "/Handlers/config.json"
};
// CREATED AN OBJECT THAT HAS KEYS AS KEYWORD NAMES OF SERVICES AND A FUNCTION THAT MAKES CHANGES TO THE PARTICULAR SERVICE FILES
let processFunctions = {
  // FUNCTION TO MAKE CHANGES IWFETL config.json FILES
  iwfetl: ({ username, password, host, port, enable_ssl, enMqPw }) => {
    let filePath = path.join(config.iwfetl, filePaths.iwfetl);
    try {
      if (fileChecker([filePath])) {
        const data = require(filePath);
        if (!data.rabbitmq) data.rabbitmq = {};
        data.rabbitmq.user = username;
        data.rabbitmq.password = enMqPw || password;
        data.rabbitmq.serverip = host;
        data.rabbitmq.secure = enable_ssl;
        data.rabbitmq.port = port;
        data.rabbitmq.secure = enable_ssl;
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
          if (err) {
            logger.error(
              `An error occurred while writing changes in ${filePath}`
            );
            logger.error(err.message);
          }
        });
      } else logger.error("ETL file does not exist. Run config:init");
    } catch (e) {
      logger.error(`An error occurred while reading ${filePath}`);
      logger.error(e.message);
    }
  },
  // FUNCTION TO MAKE CHANGES TO JOBNODERED config.json FILE
  jobnodered: ({ username, password, host, port, enable_ssl, enMqPw }) => {
    try {
      let filePath = path.join(config.jobnodered, filePaths.jobnodered);
      if (fileChecker([filePath])) {
        const data = require(filePath);
        if (!data.mqconnections) data.mqconnections = {};
        if (!data.mqconnections.rabbitmq) data.mqconnections.rabbitmq = {};
        data.mqconnections.rabbitmq.username = username;
        data.mqconnections.rabbitmq.password = enMqPw || password;
        data.mqconnections.rabbitmq.serverip = host;
        data.mqconnections.rabbitmq.secure = enable_ssl;
        data.mqconnections.rabbitmq.port = port;
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
          if (err) {
            logger.error(
              `An error occurred while writing changes in ${filePath}`
            );
            logger.error(err.message);
          }
        });
      } else
        logger.error(
          "job-center-node-red file does not exist. Run config:init"
        );
    } catch (e) {
      logger.error(`An error occurred while reading ${filePath}`);
      logger.error(e.message);
    }
  },
  // FUNCTION TO MAKE CHANGES TO NRESTSERVER dbconfig.json FILE
  nrestserver: ({ username, password, host, enable_ssl, port, enMqPw }) => {
    try {
      let filePath = path.join(
        config.nrestserver,
        "/fundsnservices/dbconfig.json"
      );
      if (fileChecker([filePath])) {
        const data = require(filePath);
        if (!data.rabbitmq) data.rabbitmq = {};
        data.rabbitmq.user = username;
        data.rabbitmq.password = enMqPw || password;
        data.rabbitmq.serverip = host;
        data.rabbitmq.secure = enable_ssl;
        data.rabbitmq.port = port;
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
          if (err) {
            logger.error(
              `An error occurred while writing changes in ${filePath}`
            );
            logger.error(err.message);
          }
        });
      } else logger.error("nrest-server file does not exist. Run config:init");
    } catch (e) {
      logger.error(`An error occurred while reading ${filePath}`);
      logger.error(e.message);
    }
  },
  // FUNCTION TO MAKE CHANGES NODECONSUMER config.json FILE
  nodeconsumer: ({ username, password, port, host, enable_ssl, enMqPw }) => {
    try {
      let filePath = path.join(config.nodeconsumer, filePaths.nodeconsumer);
      if (fileChecker([filePath])) {
        const data = require(filePath);
        if (!data.rabbitmq) data.rabbitmq = {};
        data.rabbitmq.username = username;
        data.rabbitmq.password = enMqPw || password;
        data.rabbitmq.serverip = host;
        data.rabbitmq.secure = enable_ssl;
        data.rabbitmq.port = port;
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
          if (err) {
            logger.error(
              `An error occurred while writing changes in ${filePath}`
            );
            logger.error(err.message);
          }
        });
      } else
        logger.error("funds-node-consumer file does not exist. Run config:init");
    } catch (e) {
      logger.error(`An error occurred while reading ${filePath}`);
      logger.error(e.message);
    }
  },
  // FUNCTION TO MAKE CHANGES WARFILE sysconfig.jds FILE
  // warfile: ({ username, password, host, enable_ssl, port }) => {
  //   let filePath = path.join(config.warfile, filePaths.warfile);
  //   if (fileChecker([filePath])) {
  //     fs.readFile(
  //       filePath,
  //       { encoding: "utf-8", flag: "r" },
  //       async (err, data) => {
  //         if (err) {
  //           logger.error(`An error occurred while reading ${filePath}`);
  //           logger.error(err.message);
  //         } else {
  //           if (data.search(/iwebz.queue_details(.*)}/g) !== -1)
  //             data = data.replace(
  //               /iwebz.queue_details(.*)}/g,
  //               "iwebz.queue_details = " +
  //                 JSON.stringify({
  //                   qhost: host,
  //                   quname: username,
  //                   qpwd: password,
  //                   secure: enable_ssl,
  //                   port: port,
  //                 })
  //             );
  //           else
  //             data =
  //               data +
  //               "\niwebz.queue_details = " +
  //               `${JSON.stringify({
  //                 qhost: host,
  //                 quname: username,
  //                 qpwd: password,
  //                 secure: enable_ssl,
  //                 port: port,
  //               })};`;
  //           fs.writeFile(filePath, data, (err) => {
  //             if (err) {
  //               logger.error(
  //                 `An error occurred while writing changes in ${filePath}`
  //               );
  //               logger.error(err.message);
  //             }
  //           });
  //         }
  //       }
  //     );
  //   } else logger.error("iconfig.jds file does not exist. Run config:init");
  // },
};

// MAIN COMMAND FUNCTION
class MqCommand extends Command {
  async run() {
    // CHECKING IF CONFIG FILES EXIST OR NOT
    if (config) {
      // CALLING FILE CHECKER FUNCTION WHICH CHECKS THAT ALL THE FILES BEING USED EXIST ARE READABLE AND WRITEABLE
      // THE SERVICE FILES FUNCTION GIVES BACK US AN ARRAY OF ABSOLUTE PATH FROM THE ROOT DIRECTORY FOR DIFFERENT SERVICE FILES
      if (fileChecker(serviceFiles(config, filePaths))) {
        // PROMPTING THE USER FOR MQ INPUTS
        let info = await inquirer.prompt([
          {
            message: "Enter rabbitmq username",
            name: "username",
            validate: (res) => {
              if (res === "") return "Enter a value";
              return true;
            },
          },
          {
            message: "Enter rabbitmq password",
            name: "password",
            type: "password",
            validate: (res) => {
              if (res === "") return "Enter a value";
              return true;
            },
          },
          {
            message: "Enter rabbitmq host",
            name: "host",
            validate: (res) => {
              if (res === "") return "Enter a value";
              return true;
            },
          },
          {
            message: "Enter rabbitmq port",
            name: "port",
            validate: (res) => {
              if (!/^\d+$/.test(res)) return "Enter a number";
              return true;
            },
            default: 5672,
          },
          {
            message: "Enable SSL",
            name: "enable_ssl",
            type: "list",
            choices: ["Yes", "No"],
          },
          {
            message: "Would you like to encrypt sensitive data?",
            name: "choice_encrypt",
            type: "list",
            choices: ["Yes", "No"],
          },
        ]);
        info.enable_ssl = info.enable_ssl === "Yes";

        info.enMqPw = null;
        if (info.choice_encrypt === "Yes") {
          // Encrypt function
          encrypt = require('../../utils/encrypt')(path.join(__dirname, '../../../', 'keys/public.pem'));
          info.enMqPw = encrypt(info.password)
        }

        // MAPPING OVER THE CONFIG OBJECT TO CALL CHANGES ON ONLY THOSE SERVICES THAT HAVE BEEN INITIALIZED AND THEIR PATHS ARE PRESENT IN config.json
        Object.keys(config).forEach((service) => {
          if (processFunctions[service]) processFunctions[service](info);
        });
        logger.info("MQ Initialisation Successful");
        return {status:"success"}
      }
    } else logger.error("Config file not found. Run config:init command");
  }
}

MqCommand.description = `Initialize the rabbitMq config files. For any errors reinitialize using config:init command`;
module.exports = MqCommand;
