// PACKAGES USED
const { Command } = require('@oclif/core');
const inquirer = require('inquirer');
const fs = require('fs');
const path = require('path');

// UTILITIES
const logger = require("../../utils/logger");
const { fileChecker, serviceFiles } = require("../../utils/fileChecker");

// CHECKING IF CONFIG FILE EXISTS AND REQUIRING THE DATA IN IT
let config;
if (fs.existsSync(__dirname.slice(0, __dirname.indexOf("src")) + "config.json"))
  config = require("../../../config.json");
// SERVICES WHERE THE PORT HAS TO BE INITIALIZED AND THEIR PATHS
const services = [
    "appserver",
    "notificationcenter",
    "servicecenter",
    "jobcenter"
  ],
  filePaths = {
    appserver: "/standalone/configuration/standalone.xml",
    nrestserver: "/fundsnservices/dbconfig.json",
    reporting: "app/config/.env",
    ncenter: "dist/server/.env",
    cred_auth: "/modules/config.json",
  };

// CREATED AN OBJECT THAT HAS KEYS AS KEYWORD NAMES OF SERVICES AND A FUNCTION THAT MAKES CHANGES TO THE PARTICULAR SERVICE FILES
const processFunctions = {
  // FUNCTION TO MAKE CHANGES TO JBOSS standalone.xml SERVER FILES
  appserver: (port) => {
    let filePath = path.join(config.appserver, filePaths.appserver);
    if (fileChecker([filePath]))
      fs.readFile(
        filePath,
        { encoding: "utf-8", flag: "r" },
        async (err, data) => {
          if (err) {
            logger.error(`An error occurred while reading ${filePath}`);
            logger.error(err.message);
          } else {
            data = data.replace(
              /<socket-binding name="http" port="(?:[^>]*)"\/>/g,
              `<socket-binding name="http" port="${port}"/>`
            );
            fs.writeFile(filePath, data, (err) => {
              if (err) {
                logger.error(
                  `An error occurred while writing changes in ${filePath}`
                );
                logger.error(err.message);
              }
            });
          }
        }
      );
    else logger.error("Appserver file does not exist. Run config:init");
  },
  // FUNCTION TO MAKE CHANGES TO NRESTSERVER dbconfig.json FILE
  nrestserver: (port) => {
    let filePath = path.join(config.nrestserver, filePaths.nrestserver);
    try {
      if (fileChecker([filePath])) {
        const data = require(filePath);
        data.PORT = port;
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
          if (err) {
            logger.error(
              `An error occurred while writing changes in ${filePath}`
            );
            logger.error(err.message);
          }
        });
      } else logger.error("nrest-server file does not exist. Run config:init");
    } catch (e) {
      logger.error(`An error occurred while reading ${filePath}`);
      logger.error(e.message);
    }
  },
  // FUNCTION TO MAKE CHANGES TO NCENTER .env FILE
  ncenter: (port) => {
    let filePath = path.join(config.ncenter, filePaths.ncenter);
    if (fileChecker([filePath])) {
      fs.readFile(
        filePath,
        { encoding: "utf-8", flag: "r" },
        async (err, data) => {
          if (err) {
            logger.error(`An error occurred while reading ${filePath}`);
            logger.error(err.message);
          }
          if (data.search(new RegExp("NODE_PORT\\s*?=\\s*?(.*)", "g")) !== -1)
            data = data.replace(
              new RegExp("NODE_PORT\\s*?=\\s*?(.*)", "g"),
              `NODE_PORT=${port}`
            );
          else data = data + `\nNODE_PORT=${port}`;
          fs.writeFile(filePath, data, (err) => {
            if (err) {
              logger.error(
                `An error occurred while writing changes in ${filePath}`
              );
              logger.error(err.message);
            }
          });
        }
      );
    } else logger.error(filePath + " file does not exist. Run config:init");
  },
  // FUNCTION TO MAKE CHANGES TO REPORTING SERVER .env FILE
  // reporting: (port) => {
  //   let filePath = path.join(config.reporting, filePaths.reporting);
  //   if (fileChecker([filePath])) {
  //     fs.readFile(
  //       filePath,
  //       { encoding: "utf-8", flag: "r" },
  //       async (err, data) => {
  //         if (err) {
  //           logger.error(`An error occurred while reading ${filePath}`);
  //           logger.error(err.message);
  //         }
  //         if (data.search(new RegExp("PORT\\s*?=\\s*?(.*)", "g")) !== -1)
  //           data = data.replace(
  //             new RegExp("PORT\\s*?=\\s*?(.*)", "g"),
  //             `PORT=${port}`
  //           );
  //         else data = data + `\nPORT=${port}`;
  //         fs.writeFile(filePath, data, (err) => {
  //           if (err) {
  //             logger.error(
  //               `An error occurred while writing changes in ${filePath}`
  //             );
  //             logger.error(err.message);
  //           }
  //         });
  //       }
  //     );
  //   } else logger.error(filePath + " file does not exist. Run config:init");
  // },
  // FUNCTION TO MAKE CHANGES TO CRED AUTH config.json FILE
  // cred_auth: (port) => {
  //   let filePath = path.join(config.cred_auth, filePaths.cred_auth);
  //   try {
  //     if (fileChecker([filePath])) {
  //       const data = require(filePath);
  //       data.port = port;
  //       fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
  //         if (err) {
  //           logger.error(
  //             `An error occurred while writing changes in ${filePath}`
  //           );
  //           logger.error(err.message);
  //         }
  //       });
  //     } else logger.error("nrest-server file does not exist. Run config:init");
  //   } catch (e) {
  //     logger.error(`An error occurred while reading ${filePath}`);
  //     logger.error(e.message);
  //   }
  // },
  // FUNCTION TO MAKE CHANGES TO NGNIX FILE
  ngnix: (defaultPorts, newPorts) => {
    let filePath = path.join(config.ngnix);
    if (fileChecker([filePath])) {
      fs.readFile(
        filePath,
        { encoding: "utf-8", flag: "r" },
        async (err, data) => {
          if (err) {
            logger.error(`An error occurred while reading ${filePath}`);
            logger.error(err.message);
          }
          services.map((service) => {
            if (defaultPorts[service] !== newPorts[service]) {
              let regex = new RegExp(
                `upstream\\s*?${service}\\s*?{\\s*?\\n\\s*ip_hash;\\s*?\\n\\s*server\\s*?(.*)\\s*?:\\s*?(.*)\\s*?;`,
                "gm"
              );
              // logger.error(`newPorts ${JSON.stringify(newPorts)}`)
              let newData = regex
                .exec(data)[0]
                .replace(/:(.*);/g, `:${newPorts[service]};`);
              data = data.replace(regex, newData);
            }
          });
          fs.writeFile(filePath, data, (err) => {
            if (err) {
              logger.error(
                `An error occurred while writing changes in ${filePath}`
              );
              logger.error(err.message);
            }
          });
        }
      );
    } else logger.error(filePath + " file does not exist. Run config:init");
  },
};

// FUNCTION TO GET PORTS THAT HAVE ALREADY BEEN SEEN IT IN THE CONFIG.JSON FILE
const getDefaultPorts = () => {
  let filePath = path.join(config.ngnix),
    res = {};
  try {
    let data = fs.readFileSync(filePath, { encoding: "utf-8", flag: "r" });
    services.map((service) => {
      let regex = new RegExp(
        `upstream\\s*?${service}\\s*?{\\s*?\\n\\s*ip_hash;\\s*?\\n\\s*server\\s*?(.*)\\s*?:\\s*?(.*)\\s*?;`,
        "gm"
      );
      let temp = regex.exec(data);
      res[service] = temp ? parseInt(temp[2]) : "";
    });
    return res;
  } catch (err) {
    logger.info(`An error occurred while reading ${filePath}`);
    logger.error(err.message);
  }
};

class PortCommand extends Command {
  async run() {
    if (config) {
      if (fileChecker([...serviceFiles(config, filePaths), config.ngnix])) {
        let defaultPorts = getDefaultPorts();
        let inputs = await inquirer.prompt([
          {
            message: "Enter jboss/wildfly port",
            name: "appserver",
            default: defaultPorts["appserver"],
            validate: (res) => {
              if (res === "") return "Enter a value";
              if (!/^\d+$/.test(res)) return "Enter a number";
              return true;
            },
          },
          {
            message: "Enter node restserver microservice port",
            name: "servicecenter",
            default: defaultPorts["servicecenter"],
            validate: (res) => {
              if (res === "") return "Enter a value";
              if (!/^\d+$/.test(res)) return "Enter a number";
              return true;
            },
          },
          {
            message: "Enter notification center microservice port",
            name: "notificationcenter",
            default: defaultPorts["notificationcenter"],
            validate: (res) => {
              if (res === "") return "Enter a value";
              if (!/^\d+$/.test(res)) return "Enter a number";
              return true;
            },
          },
          {
            message: "Enter jobnodered microservice port",
            name: "jobcenter",
            default: defaultPorts["jobcenter"],
            validate: (res) => {
              if (res === "") return "Enter a value";
              if (!/^\d+$/.test(res)) return "Enter a number";
              return true;
            },
          },
          // {
          //   message: "Enter metabase port",
          //   name: "metabase",
          //   default: defaultPorts["metabase"],
          //   validate: (res) => {
          //     if (res === "") return "Enter a value";
          //     if (!/^\d+$/.test(res)) return "Enter a number";
          //     return true;
          //   },
          // },
          // {
          //   message: "Enter reporting server microservice port",
          //   name: "reporting",
          //   default: defaultPorts["reporting"] || "",
          //   validate: (res) => {
          //     if (res === "") return "Enter a value";
          //     if (!/^\d+$/.test(res)) return "Enter a number";
          //     return true;
          //   },
          // },
          // {
          //   message: "Enter credauth microservice port",
          //   name: "cred_auth",
          //   default: defaultPorts["cred_auth"],
          //   validate: (res) => {
          //     if (res === "") return "Enter a value";
          //     if (!/^\d+$/.test(res)) return "Enter a number";
          //     return true;
          //   },
          // },
        ]);
        inputs["rest"] = inputs["app"];
        processFunctions["appserver"](inputs.appserver);
        if (config.nrestserver) processFunctions["nrestserver"](inputs.servicecenter);
        if (config.ncenter) processFunctions["ncenter"](inputs.notificationcenter);
        if (config.reporting) processFunctions["reporting"](inputs.reporting);
        if (config.cred_auth) processFunctions["cred_auth"](inputs.cred_auth);
        processFunctions["ngnix"](defaultPorts, inputs);
        logger.info("Port Initialisation Successful");
      }
    } else logger.error("Config file not found. Run config:init command");
  }
}

PortCommand.description = `Change the port for nrest and App Server file`;
module.exports = PortCommand;
