const { Command } = require("@oclif/core");
const { execSync } = require("child_process");
const inquirer = require("inquirer");
const path = require("path");
const fs = require("fs");
const AdmZip = require('adm-zip');
const logger = require("../../utils/logger");

class HeapDumpCommand extends Command {
    async run() {
        const dateTime = new Date().getTime();
        const filename = `heapdump_${dateTime}.hprof`;
        const filepath = path.join(process.cwd(), filename);
        const compressed = path.join(process.cwd(), `heapdump_${dateTime}.zip`);
        try {
            let command, output, thread, zip;
            const { javaJdkPath } = await inquirer.prompt({
                name: "javaJdkPath",
                message: "Enter the path for JAVA JDK folder",
                type: "input",
                validate: (res) => {
                    if (fs.existsSync(res) && fs.existsSync(path.join(res, "/bin/"))) {
                        return true;
                    }
                    return "Invalid JAVA JDK folder directory";
                },
                filter: (res) => res.split("\\").join("/"),
            });
            command = `"${javaJdkPath}/bin/jps" -l`;
            output = execSync(command).toString();
            thread = output.split("\n");
            thread = thread.filter(
                (thread) =>
                    thread.toLowerCase().includes("jboss") ||
                    thread.toLowerCase().includes("wildfly")
            )[0];
            if (thread) {
                command = `"${javaJdkPath}/bin/jcmd" ${thread.slice(0, thread.indexOf(" "))} GC.heap_dump ${filepath}`;
                output = execSync(command);
                zip = new AdmZip();
                zip.addLocalFile(filepath)
                zip.writeZip(compressed)
                if (fs.existsSync(filepath)) {
                    fs.unlinkSync(filepath);
                }
                logger.info(`Heap dump file created at ${compressed}`)
            }
            else {
                throw new Error("No jboss or wildFly process found");
            }
        } catch (error) {
            logger.error(error);
        }
    }
}

HeapDumpCommand.description = `Creates a heap dump of the wildfly/jboss process`;
module.exports = HeapDumpCommand;
