// PACKAGES USED
const { Command } = require('@oclif/core');
const { execSync } = require('child_process');
const inquirer = require('inquirer');
const path = require('path');
const fs = require('fs');

const { logger } = require("../../utils/logger");

// MAIN COMMAND LINE FUNCTION
class ThreadDumpCommand extends Command {
    async run() {
        try {
            const { javaJdkPath } = await inquirer.prompt({
                name: 'javaJdkPath',
                message: 'Enter the path for JAVA JDK folder',
                type: 'input',
                validate: (res) => fs.existsSync(res) && fs.existsSync(path.join(res, '/bin/')),
                filter: (res) => res.split('\\').join('/'),
            });
            let command = `"${javaJdkPath}/bin/jps" -l`;
            let output = execSync(command).toString();
            let thread = output.split('\n');
            thread = thread.filter((thread) => thread.toLowerCase().includes('jboss') || thread.toLowerCase().includes('wildfly'))[0];
            if (thread) {
                let file = `threaddump_${new Date().getTime()}.txt`;
                command = `"${javaJdkPath}/bin/jstack" -l ${thread.slice(0, thread.indexOf(' '))} > ${path.join(process.cwd(), file)}`;
                output = execSync(command);
                logger.info(`Thread dump file location: ${path.join(process.cwd(), file)}`);
            } else {
                throw new Error('No JBoss or WildFly process found');
            }
        } catch (error) {
            logger.error(error);
        }
    }
}

ThreadDumpCommand.description = `Creates a thread dump of the current process`;
module.exports = ThreadDumpCommand;
