// PACKAGES USED
const { Command } = require('@oclif/core');
const fs = require('fs');
const inquirer = require('inquirer');
const path = require('path');
const AmdZip = new require('adm-zip');
let zip = new AmdZip();

// UTILITIES
const { logger } = require("../../utils/logger");
const { fileChecker, serviceFiles } = require("../../utils/fileChecker");

// CHECKING IF CONFIG FILE EXISTS AND REQUIRING THE DATA IN IT
let config;
if (fs.existsSync(__dirname.slice(0, __dirname.indexOf('src')) + 'config.json')) config = require('../../../config.json');
let filePaths = {
  iwfetl: '/Logs',
  jobnodered: '/Logs',
  nodeconsumer: '/Logs',
  nrestserver: '/Logs',
  appserver: '/standalone/log',
  warfile: '/log',
  fixInterface2: '/FIXTrading/log',
  fixInterface4: '/FIXTrading/log',
},
  zipFileStructure = {
    iwfetl: config && config.iwfetl ? `/${path.parse(config.iwfetl).base}/Logs` : '',
    jobnodered: config && config.jobnodered ? `/${path.parse(config.jobnodered).base}/Logs` : '',
    nodeconsumer: config && config.nodeconsumer ? `${path.parse(config.nodeconsumer).base}/Logs` : '',
    nrestserver: config && config.nrestserver ? `/${path.parse(config.nrestserver).base}/Logs` : '',
    fixInterface4: config && config.fixInterface4 ? `/${path.parse(config.fixInterface4).base}/FIXTrading/log` : '',
    fixInterface2: config && config.fixInterface2 ? `/${path.parse(config.fixInterface2).base}/FIXTrading/log` : '',
    appserver: config && config.appserver ? `/${path.parse(config.appserver).base}/standalone/log` : '',
    warfile: config && config.warfile ? `/${path.parse(config.appserver).base}/standalone/deployments/${path.parse(config.warfile).base}/log` : '',
  };

class ExtractLogsCommand extends Command {
  async run() {
    if (config) {
      if (fileChecker(serviceFiles(config, filePaths))) {
        let outputPath = await inquirer.prompt({
          name: 'dir',
          message: 'Enter log dump path',
          default: process.cwd(),
          validate: (res) => fileChecker([res], 'returnMessage'),
          filter: (res) => res.split('\\').join('/'),
        });
        outputPath = path.join(outputPath.dir, `Logs_${new Date().getTime()}.zip`);
        Object.keys(config).forEach((service) => {
          if (filePaths[service]) zip.addLocalFolder(path.join(config[service], filePaths[service]), zipFileStructure[service]);
        });
        zip.writeZip(outputPath);
        logger.info(`Logs dumped successfully. Zip path: ${outputPath}`);
      }
    } else logger.error('Config file not found. Run config:init command');
  }
}

ExtractLogsCommand.description = `Store system information in a file`;
module.exports = ExtractLogsCommand;
