const { Command } = require("@oclif/core");
const logger = require('../../utils/logger');
const path = require('path')
const fs = require('fs')

const decrypt = require('../../utils/decryptOld')
const runCommands = require('../../utils/internalCommands');

let filePaths = {
    nrestserver: "/fundsnservices/dbconfig.json",
    nodeconsumer: "/Handlers/config.json",
    // iwfetl: "custom/config.json"
}

function checkforexistances(key) {
    let config = require('../../../config.json')
    let filePath = "";
    let result = "";

    if (key in config) {
        filePath = path.join(config[key], filePaths[key])
        let publickeyPath = path.join(filePath, "../", "public.pem")
        result = fs.existsSync(filePath) && fs.existsSync(publickeyPath)
        return ({ isFileExist: result, filePath: filePath, publicPath: publickeyPath, name: key, keyPath: path.join(config.nrestserver, '../keys/private.pem').replace(/\\/g, '/') })
    }
}

async function runCommandsForDecryption(responseObject) {

    if (responseObject.nrestserver.isFileExist) {
        let keyResult; let mqData; let productData; let databaseData

        const { keyPath } = responseObject.nrestserver
        /**below code for db config file decryption  */
        const isdecrypt = decrypt(responseObject.nrestserver.filePath, responseObject.nrestserver.publicPath, keyPath,"Nrestserver config")
        if (isdecrypt.status != "success") {
            return false
        }
        /**below code for config file decryption  */
        if (responseObject.nodeconsumer.isFileExist) {
            let result = decrypt(responseObject.nodeconsumer.filePath, responseObject.nodeconsumer.publicPath, keyPath,"Nodeconsumer config")
            if (result.status != "success") {
                return false
            }
        }
        /**below code for etl config file decryption */
        // if (responseObject.iwfetl.isFileExist) {
        //     let result = decrypt(responseObject.iwfetl.filePath, responseObject.iwfetl.publicPath)
        //     if (result.status != "success") {
        //         return false
        //     }
        // }

        keyResult = await runCommands.runKeyGenrateCommandInternally()
        if (keyResult.status != "success") {
            return false
        }
        mqData = await runCommands.runmqcommandinternally()
        if (mqData.status != "success") {
            return false
        }
        productData = await runCommands.runproductcommandinternally()
        if (productData.status != "success") {
            return false
        }
        databaseData = await runCommands.rundbcommandinternally()
        if (databaseData.status != "success") {
            return false
        }
        return true
    } else {
        return false
    }
}

class DecryptCommand extends Command {
    async run() {

        if (fs.existsSync(__dirname.slice(0, __dirname.indexOf("src")) + "config.json")) {
            let responseObject = {}
            for (let key in filePaths) {
                let response = await checkforexistances(key)
                responseObject[key] = response
            }

            let output = await runCommandsForDecryption(responseObject)
            if (output) {
                logger.info('Database password has been encrypted successfully.')
            } else {
                logger.info('please check file exist or not')
            }

        } else {
            logger.error(`Can not generate Decryption files! please check config file exist or not`)
        }
    }
}

DecryptCommand.description = `Decrypt dbconfig file using public key`;
module.exports = DecryptCommand;
