const { Command } = require('@oclif/core');
const path = require('path');
const { prompt } = require('enquirer');
const fs = require('fs');
const Listr = require('listr');
const chalk = require('chalk');
const execPromise = require('util').promisify(require('child_process').exec);
const os = require('os');
const { readConfig } = require('../../utils/utils');

const { initLogger, setConsoleLog, CliError, HOME_DIR } = require(path.join(__dirname, '../../utils/logger.js'));
const { errorHandler, checkMemory } = require(path.join(__dirname, '../../utils/utils.js'));
const logger = initLogger('product');
setConsoleLog(Command);
const { initIwf, initInput } = require(path.join(__dirname, '../../utils/helper/init.js'));
const { initUninstall } = require(path.join(__dirname, '../../utils/helper/init-helper.js'));
const { prerequisite, prerequisiteInput, dependenciesUninstall } = require(path.join(__dirname, '../../utils/helper/prerequisite.js'));
const { microservices, microserviceInput, microservicesUninstallation } = require(path.join(__dirname, '../../utils/helper/microservices.js'));
const { mysqlConfig } = require(path.join(__dirname, '../../utils/helper/mysql-helper.js'));
const { mailingEngine, mailingEngineInput, } = require(path.join(__dirname, '../../utils/helper/mailing-engine.js'));
const { mailingEngineUninstall } = require(path.join(__dirname, '../../utils/helper/mailing-engine-helper.js'));
const { apps, appsInput } = require(path.join(__dirname, '../../utils/helper/apps.js'));
const { appsUninstall } = require(path.join(__dirname, '../../utils/helper/apps-helper.js'));


class InstallCommand extends Command {
  async run() {
    try {
      let config = {};
      let appsInputValues;

      const osDetails = {
        type: os.type(),
        version: os.type() === 'Linux' ? /PRETTY_NAME="([^"]+)"/.exec(fs.readFileSync('/etc/os-release').toString())[1] : os.version(),
        platform: os.platform()
      }
      logger.info(`Installation... and home directory: ${HOME_DIR}`);
      if (!fs.existsSync(path.join(__dirname, '../../../product_cfg.json'))) {
        const configArray = JSON.stringify(
          {
            'os': osDetails,
            'init': {
              'path': ``,
              'isInstalled': false,
              'installationMode': '',
            },
            'dependencies': {
              'java': {
                'path': '',
                'isInstalled': false,
              },
              'oracleInstantClient': {
                'path': '',
                'isInstalled': false,
              },
              'rabbitMq': {
                'path': '',
                'isInstalled': false,
              },
              'nginx': {
                'path': '',
                'isInstalled': false,
              },
              'python': {
                'path': '',
                'isInstalled': false,
              },
              'MySql': {
                'path': '',
                'isInstalled': false,
              },
              'pm2': {
                'path': '',
                'isInstalled': false,
              },
            },
            'services': {
              'etl': {
                'path': '',
                'isInstalled': false,
              },
              'jobcenter': {
                'path': '',
                'isInstalled': false,
              },
              'metabase': {
                'path': '',
                'isInstalled': false,
              },
              'NCenter': {
                'path': '',
                'isInstalled': false,
              },
              'nodeconsumer': {
                'path': '',
                'isInstalled': false,
              },
              'funds_sc': {
                'path': '',
                'isInstalled': false,
              },
              'excel-utility': {
                'path': '',
                'isInstalled': false,
              },
              'dbmodel': {
                'path': '',
                'isInstalled': false,
              },
              'cred-auth': {
                'path': '',
                'isInstalled': false,
              },
              'auto-upload-csv': {
                'path': '',
                'isInstalled': false,
              },
              'automated-upload': {
                'path': '',
                'isInstalled': false,
              },
              'applicationServer': {
                'type': '',
                'path': '',
                'isInstalled': false,
                'warfile': {
                  'type': '',
                  'path': '',
                  'isInstalled': false,
                },
              },
            },
            'mailingEngine': {
              'path': '',
              'isInstalled': false,
            },
            'apps': {
              'app-builder': {
                'path': '',
                'isInstalled': false,
              },
              'app-components': {
                'path': '',
                'isInstalled': false,
              },
              'Custom_dashboard': {
                'path': '',
                'isInstalled': false,
              },
              'Exception': {
                'path': '',
                'isInstalled': false,
              },
              'FIS': {
                'path': '',
                'isInstalled': false,
              },
              'Historical': {
                'path': '',
                'isInstalled': false,
              },
              'micro-frontend-legacy-setup': {
                'path': '',
                'isInstalled': false,
              },
              'NPA': {
                'path': '',
                'isInstalled': false,
              },
              'Operations': {
                'path': '',
                'isInstalled': false,
              },
              'page-builder': {
                'path': '',
                'isInstalled': false,
              },
              'Position-Dashboard': {
                'path': '',
                'isInstalled': false,
              },
              'Returns_dashboard': {
                'path': '',
                'isInstalled': false,
              },
              'Risk Management': {
                'path': '',
                'isInstalled': false,
              },
              'Trader Desk': {
                'path': '',
                'isInstalled': false,
              },
            },
          },
          null,
          2
        );
        fs.writeFileSync(path.join(__dirname, `../../../product_cfg.json`), configArray);
      }


      const checkAvailMemory = new Listr([
        {
          title: 'Checking free disk space',
          task: async () => {
            const diskSpace = await checkMemory();
            if (diskSpace.freeGB < 10) {
              this.console.warn('The installation files require 5.0GB of space');
              throw new CliError('Storage full! Make sure you have 10 GB available space on disk to proceed.');
            }
          }
        }
      ]);
      await checkAvailMemory.run().catch((e) => {
        logger.error(`Not Enough Memory left ${e.stack ? e.stack : e}`);
        throw new Error(`Not Enough Memory: Make sure you have 10 GB available space on disk to proceed`)
      });

      config = readConfig();

      const dependencies = await prompt([
        {
          type: 'select',
          name: 'all',
          message: 'Would you like to download and install all the Installation Files ? ',
          choices: ['yes', 'no'],
        },
      ]);

      const initInputValues = await initInput(dependencies.all);

      const download = new Listr([
        {
          title: 'Initializing Installation Process',
          enabled: () => {
            if (dependencies.all === 'yes') {
              return true;
            } else if (dependencies.all === 'no' && initInputValues.init.answer === 'yes') {
              return true;
            }
            return false;
          },
          task: async () => {
            return await initIwf(initInputValues.installationType, initInputValues.directoryForInitPath, initInputValues.gitInput);
          },
        },
      ]);
      await download.run().catch(async (e) => {
        logger.error(`Error: Error occurred during download : ${e.stack ? e.stack : e}`);
        this.console.log('Rolling back changes to maintain data integrity')
        // const rollback = new Listr([
        //   {
        //     title: 'Init',
        //     task: async () => await initUninstall(config.init),
        //   }
        // ]);
        // await rollback.run().catch((e) => {
        //   logger.error(`Error occurred while rolling back : ${e.stack ? e.stack : e}`);
        // });
        throw new Error('Installation failed and rollback completed successfully');
      });

      const directory = config.init.path;

      if (config.init.isInstalled) {

        const prerequisiteInputValues = await prerequisiteInput(dependencies.all);
        const microserviceInputValues = await microserviceInput(directory, dependencies.all);
        const mailing_engine = await mailingEngineInput(dependencies.all);
        if (prerequisiteInputValues.hasOwnProperty('prerequisiteDependencies')) {
          appsInputValues = await appsInput(directory, dependencies.all, prerequisiteInputValues.prerequisiteDependencies.all === 'yes' || prerequisiteInputValues.prerequisitesToInstall.includes('Nginx'));
        }
        
        logger.info(`APPS inputs: ${JSON.stringify(appsInputValues)}`);
        if (appsInputValues === undefined) {
          try {
            const appsCondition = await execPromise('sudo systemctl status nginx');

            if (appsCondition.stdout.includes('Active: active (running)')) {
              appsInputValues = await appsInput(directory, dependencies.all, true);
            } else {
              appsInputValues = await appsInput(directory, dependencies.all, false);
            }
          } catch (e) {
            logger.error(`error during appsInput : ${e}`);
            appsInputValues = await appsInput(directory, dependencies.all, false);
          }
        }
        
        logger.info(`All inputs taken... prerequisite inputs: ${JSON.stringify(prerequisiteInputValues)}`);

        const tasks = new Listr([
          {
            title: 'Installing Dependencies',
            enabled: () => {
              if (dependencies.all === 'yes') {
                return true;
              } else if (dependencies.all === 'no' && prerequisiteInputValues.preReq.answer === 'yes') {
                return true;
              }
              return false;
            },
            task: async () => {
              return await prerequisite(prerequisiteInputValues.prerequisiteDependencies, directory, prerequisiteInputValues.prerequisitesToInstall);
            },
          },
          {
            title: 'Installing Services',
            enabled: () => {
              if (dependencies.all === 'yes') {
                return true;
              } else if (dependencies.all === 'no' && microserviceInputValues.microInp.answer === 'yes') {
                return true;
              }
              return false;
            },
            task: async () => {
              return await microservices(directory, microserviceInputValues.microserviceDependencies, microserviceInputValues.microservicesToInstall, microserviceInputValues.appServerChoice);
            },
          },
          // {
          //   title: 'Installing Mailing-Engine',
          //   enabled: () => {
          //     if (dependencies.all === 'yes') {
          //       return true;
          //     } else if (dependencies.all === 'no' && mailing_engine === 'yes') {
          //       return true;
          //     }
          //     return false;
          //   },
          //   task: async () => {
          //     return await mailingEngine(directory);
          //   },
          // },
          {
            title: 'Installing Apps',
            skip: async () => {
              if (dependencies.all === 'yes') {
                return false;
              }
              const nginxStatus = await execPromise('sudo systemctl status nginx');

              if (nginxStatus.stdout.includes('active (running)')) {
                return false;
              }
              return 'nginx is not running or not installed';
            },
            enabled: () => {
              if (dependencies.all === 'yes') {
                return true;
              } else if (!(appsInputValues.appInp === undefined)) {
                if (dependencies.all === 'no' && appsInputValues.appInp.answer === 'yes') {
                  return true;
                }
              }
              return false;
            },
            task: async () => {
              return await apps(directory, appsInputValues.appsDependencies, appsInputValues.appsToInstall);
            }
          },
        ]);
        await tasks.run().catch(async (error) => {
          logger.error(`Error: Error occurred during installation of services ${error.stack ? error.stack : error}`)
          this.console.log('Rolling back changes to maintain data integrity')
          // const rollback = new Listr([
          //   {
          //     title: 'Deleting Dependencies',
          //     task: async () => await dependenciesUninstall(config.dependencies),
          //   },
          //   {
          //     title: 'Deleting Services',
          //     task: async () => await microservicesUninstallation(config.services),
          //   },
          //   {
          //     title: 'Deleting Mailing Engine',
          //     task: async () => await mailingEngineUninstall(config.mailingEngine),
          //   },
          //   {
          //     title: 'Deleting Apps',
          //     task: async () => await appsUninstall(config.apps)
          //   },
          // ]);

          // await rollback.run().catch((e) => {
          //   logger.error(`Error occurred while rolling back : ${e.stack ? e.stack : e}`);
          // });
          throw new Error('Installation failed and rollback completed successfully');
        });

        // await mysqlConfig();

        const isAnyAppInstalled = Object.values(config.apps).some((app) => app.isInstalled);
        const newConfig = Object.fromEntries(
          Object.entries({
            warfile: config.services.applicationServer.warfile.isInstalled ? config.services.applicationServer.warfile.path : '',
            nginx: config.dependencies.nginx.isInstalled ? `${config.dependencies.nginx.path}/nginx.conf` : '',
            apps: isAnyAppInstalled ? `${HOME_DIR}/credence/apps/ui` : '',
            nodeconsumer: config.services.nodeconsumer.isInstalled ? config.services.nodeconsumer.path : '',
            nrestserver: config.services.funds_sc.isInstalled ? config.services.funds_sc.path : '',
            iwfetl: config.services['etl'].isInstalled ? config.services['etl'].path : '',
            jobnodered: config.services.jobcenter.isInstalled ? config.services.jobcenter.path : '',
            ncenter: config.services.NCenter.isInstalled ? config.services.NCenter.path : '',
            // reporting: config.services['reporting-server'].isInstalled ? config.services['reporting-server'].path : '',
            // mailingEngine: config.mailingEngine.isInstalled ? config.mailingEngine.path : '',
            appserver: config.services.applicationServer.isInstalled ? config.services.applicationServer.path : '',
          }).filter(([key, value]) => value !== '')
        );
        const configObj = JSON.stringify(newConfig, null, 2);
        fs.writeFileSync(path.join(__dirname, '../../../config.json'), configObj);
        this.console.log(`Please use the following commands to configure the installed services :`);
        this.console.log(`${chalk.blueBright(`fundscli config:database`)}- To update the database connection string in all services`);
        this.console.log(`${chalk.blueBright(`fundscli config:java`)} - To update java details in jboss directory`);
        this.console.log(`${chalk.blueBright(`fundscli config:mq`)}- To update the rabbitmq details in all services`);
      } else {
        this.console.error(chalk.redBright('idf-installations directory is not present'));
      }
    } catch (error) {
      logger.error(`Error occurred due to : ${error}`);
      errorHandler(error, 'product');
    }
  }
}

InstallCommand.description = `Installs dependencies, services and applications for jboss`;

InstallCommand.flags = {};

module.exports = InstallCommand;
