const { Command } = require('@oclif/core');
const path = require('path');
const fs = require('fs');
const Listr = require('listr');
const { prompt } = require('enquirer');
const { readConfig } = require('../../utils/utils');

const { initUninstall } = require(path.join(__dirname, '../../utils/helper/init-helper.js'));
const { dependenciesUninstall } = require(path.join(__dirname, '../../utils/helper/prerequisite.js'));
const { appsUninstall } = require(path.join(__dirname, '../../utils/helper/apps-helper.js'));
const { microservicesUninstallation } = require(path.join(__dirname, '../../utils/helper/microservices.js'));
const { mailingEngineUninstall } = require(path.join(__dirname, '../../utils/helper/mailing-engine-helper.js'));
const { initLogger, setConsoleLog } = require(path.join(__dirname, '../../utils/logger.js'));
const { errorHandler } = require(path.join(__dirname, '../../utils/utils.js'));

const logger = initLogger('product');

setConsoleLog(Command);

class UninstallCommand extends Command {
  async run() {
    try {
      const config = readConfig();
      this.console.warn('This command completely wipes all installations made through fundscli');
      const confirmPrompt = await prompt([
        {
          type: 'select',
          name: 'answer',
          message: 'Are you sure you want to proceed',
          choices: ['yes', 'no'],
        },
      ]);

      if (confirmPrompt.answer === 'yes') {

        const uninstall = new Listr([
          {
            title: 'Deleting Dependencies',
            task: async () => await dependenciesUninstall(config.dependencies),
          },
          {
            title: 'Deleting Services',
            task: async () => await microservicesUninstallation(config.services),
          },
          {
            title: 'Deleting Mailing Engine',
            task: async () => await mailingEngineUninstall(config.mailingEngine),
          },
          {
            title: 'Deleting Apps',
            task: async () => await appsUninstall(config.apps)
          },
          {
            title: 'Deleting Installation Files',
            task: async () => await initUninstall(config.init),
          },
        ]);

        uninstall.run().catch((e) => {
          logger.error(`Error occurred in uninstall listr : ${e}`);
        });
      }
    } catch (error) {
      logger.error(`Error occurred due to : ${error}`);
      errorHandler(error, 'product');
    }
  }
}
UninstallCommand.description = `Uninstall IDF installations`;

UninstallCommand.flags = {};

module.exports = UninstallCommand;
