const path = require('path');

const { prompt } = require('enquirer');
const { readConfig } = require('../utils');

const { initLogger } = require(path.join(__dirname, '../logger.js'));
const { errorHandler } = require(path.join(__dirname, '../../utils/utils.js'));
const { wildflyInstallation, wildflyUninstall } = require(path.join(__dirname, '../../utils/helper/wildfly-helper.js'));
const { jbossInstallation, jbossUninstall } = require(path.join(__dirname, '../../utils/helper/jboss-helper.js'));
const { iwfWarInstallation, iwfWarUninstall } = require(path.join(__dirname, '../../utils/helper/iwf-war-helper.js'));

const logger = initLogger('product');

async function appServerInput(all) {
  try {
    let appServerInput;
    if (all === 'no') {
      appServerInput = await prompt([
        {
          type: 'select',
          name: 'answer',
          message: 'Would you like to install application server ? ',
          choices: ['yes', 'no'],
        }
      ]);
    }
    if (all === 'yes' || appServerInput.answer === 'yes') {
      const appServerValues = await prompt([
        {
          type: 'select',
          name: 'answer',
          message: 'Which Application Server do you want ? ',
          choices: ['JBOSS', 'Wildfly'],
        }
      ]);
      return { appServerValues, appServerInput: { answer: 'yes' } };
    }
    return { appServerInput: { answer: 'no' } };
  } catch (e) {
    throw new CliError(`Error occurred during taking app-server input : ${error.stack}`)
  }
}

async function appServer(directory, appServerChoice) {
  try {
    const config = readConfig();
    if (!config.services.applicationServer.isInstalled) {
      if (appServerChoice.answer === 'JBOSS') {
        await jbossInstallation(directory);
      } else if (appServerChoice.answer === 'Wildfly') {
        await wildflyInstallation(directory);
      }
      if (config.services.applicationServer.warfile.isInstalled) {
        logger.info('Funds WAR is already extracted');
      } else {
        await iwfWarInstallation(directory);
      }
    }
  } catch (error) {
    logger.error(`Error occurred : ${error}`)
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during appServer : ${error.stack}`)
  }
}

async function appServerUninstall(appServerDetails) {
  try {
    await iwfWarUninstall(appServerDetails.warfile);
    if (appServerDetails.type === 'wildfly') {
      await wildflyUninstall(appServerDetails);
    } else {
      await jbossUninstall(appServerDetails);
    }
  } catch (error) {
    logger.error(`Error occurred : ${error}`);
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during appServerUninstall : ${error.stack}`)
  }
}

module.exports = { appServer, appServerInput, appServerUninstall };
