const extract = require('extract-zip');
const fs = require('fs');
const os = require('os');
const path = require('path');
const Listr = require('listr');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');

const appsDirectory = `${HOME_DIR}/credence/apps/ui`;

async function appsInstallation(directory, all, selectedFiles = []) {
  try {
    const config = readConfig();

    if (!fs.existsSync(path.resolve(`${directory}/Apps/ui`))) {
      logger.error('Provided path for installation files is incorrect');
      throw new CliError('Provided path for installation files is incorrect')
    }
    if (!fs.existsSync(path.resolve(appsDirectory))) {
      fs.mkdirSync(appsDirectory);
    }
    if (all === 'yes') {
      try {
        const files = await fs.promises.readdir(`${directory}/Apps/ui`)
        files.forEach(file => {
          fs.copyFileSync(`${directory}/Apps/ui/${file}`, `${appsDirectory}/${file}`)
        })
      } catch (e) {
        throw new CliError(`Error occurred while copying : ${e.stack}`)
      }
      const files = fs.readdirSync(path.resolve(appsDirectory)).filter(el => el.includes('.zip'));
      logger.info(`files in apps directory : ${files}`)
      const tasks = new Listr([]);
      if (files.length > 0) {
        for (const file of files) {
          tasks.add({
            title: `${file.slice(0, -4).toUpperCase()}`,
            task: async () => {
              if (!fs.existsSync(`${appsDirectory}/${file.slice(0, -4)}`)) {
                await extract(`${appsDirectory}/${file}`, { dir: appsDirectory });
                logger.info(`${file} installed`);
                fs.rmSync(path.resolve(`${appsDirectory}/${file}`));
                config.apps[file.slice(0, -4)].path = `${appsDirectory}/${file.slice(0, -4)}`;
                config.apps[file.slice(0, -4)].isInstalled = true;
                const newConfig = JSON.stringify(config, null, 2);
                fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
              }
            }
          });
        }
      }
      return tasks;
    } else {
      const tasks = new Listr([]);
      for (const file of selectedFiles) {
        tasks.add({
          title: `${file.slice(0, -4).toUpperCase()}`,
          task: async () => {
            if (!fs.existsSync(`${appsDirectory}/${file.slice(0, -4)}`)) {
              fs.copyFileSync(`${directory}/Apps/ui/${file}`, `${appsDirectory}/${file}`);
              await extract(`${appsDirectory}/${file}`, { dir: `${appsDirectory}/`, });
              fs.rmSync(`${appsDirectory}/${file}`, {
                force: true,
              });
              config.apps[file.slice(0, -4)].path = `${appsDirectory}/${file.slice(0, -4)}`;
              config.apps[file.slice(0, -4)].isInstalled = true;
              const newConfig = JSON.stringify(config, null, 2);
              fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
            }
          }
        });
      }
      return tasks;
    }
  } catch (error) {
    logger.error(`Error in appsInstallation due to : ${error}`);
    throw new CliError(`Error in appsInstallation due to : ${error}`);
  }
}

function appsUninstall(appsDetails) {
  try {
    const config = readConfig();

    return new Listr([
      {
        title: 'Deleting Apps',
        task: () => {
          const apps = Object.keys(appsDetails);
          for (const app of apps) {
            if (appsDetails[app].isInstalled) {
              fs.rmSync(appsDetails[app].path, { recursive: true, force: true, });
              config.apps[app].path = ``;
              config.apps[app].isInstalled = false;
            } else {
              logger.info(`${app} is not installed`);
            }
          }
          fs.rmSync(appsDirectory, { recursive: true, force: true, });
          const newConfig = JSON.stringify(config, null, 2);
          fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
        }
      }
    ]);
  } catch (error) {
    logger.error(`Error in appsUninstall due to : ${error}`);
    throw new CliError(`Error in appsUninstall due to : ${error}`);
  }
}

module.exports = { appsInstallation, appsUninstall };
