const path = require('path');
const os = require('os');
const fs = require('fs');
const { prompt } = require('enquirer');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));
const { errorHandler } = require(path.join(__dirname, '../../utils/utils.js'));
const { appsInstallation } = require(path.join(__dirname, '../../utils/helper/apps-helper.js'));

const logger = initLogger('product');

async function appsInput(directory, all, isNginx) {
  try {
    let appInp;
    let appsToInstall = { apps: [] };

    if (all === 'no') {
      if (isNginx) {
        appInp = await prompt([
          {
            type: 'select',
            name: 'answer',
            message: 'Would you like to install any apps ? ',
            choices: ['yes', 'no'],
          }
        ]);
      } else {
        appInp = { answer: 'no' };
      }
    }
    if (all === 'yes' || appInp.answer === 'yes') {
      const appsDependencies = await prompt([
        {
          type: 'select',
          name: 'all',
          message: 'Would you like to install all the apps ? ',
          choices: ['yes', 'no'],
        }
      ]);
      if (appsDependencies.all === 'no') {
        const files = fs
          .readdirSync(path.resolve(`${directory}/Apps/ui`))
          .filter((el) => el.includes('.zip'))
          .map((el) => el.replace('.zip', ''));

        appsToInstall = await prompt([{
          type: 'multiselect',
          name: 'apps',
          message: 'Select the apps to install',
          choices: files,
          initial: files,
          instructions: true,
        }])

      }
      return {
        appsDependencies,
        appsToInstall: appsToInstall.apps,
        appInp,
      };
    }
    return { appInp };
  } catch (error) {
    errorHandler(error, 'product')
    throw new CliError(`Error occurred during taking apps input : ${error.stack}`)
  }
}

async function apps(directory, dependencies, appsToInstall) {
  try {
    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence/apps/ui`))) {
      fs.mkdirSync(`${HOME_DIR}/credence/apps/`);
      fs.mkdirSync(`${HOME_DIR}/credence/apps/ui/`);
    }
    const files = fs
      .readdirSync(path.resolve(`${directory}/Apps/ui`))
      .filter((el) => el.includes('.zip'))
      .map((el) => el.replace('.zip', ''));

    if (dependencies.all === 'yes') {
      const installedFiles = (await fs.promises.readdir(path.resolve(`${HOME_DIR}/credence/apps/ui`))).filter((el) => !el.includes('.'));

      function arrayEquals(a, b) {
        return Array.isArray(a) && Array.isArray(b) && a.length === b.length && a.every((val, index) => val === b[index]);
      }

      if (fs.existsSync(path.resolve(`${HOME_DIR}/credence/apps/ui`)) && arrayEquals(files, installedFiles)) {
        logger.info('Apps are already installed');
      } else {
        return await appsInstallation(directory, dependencies.all);
      }
    } else {
      const selectedKeys = appsToInstall.map((el) => el + '.zip');
      return await appsInstallation(directory, dependencies.all, selectedKeys);
    }
  } catch (error) {
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during apps : ${error.stack}`)
  }
}

module.exports = { apps, appsInput };
