const fs = require('fs');
const simpleGit = require('simple-git');
const path = require('path');
const extract = require('extract-zip');
const os = require('os');
const Listr = require('listr');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');

const git = simpleGit.default();

async function cloningDirectory(directory, gitInput) {
  try {
    const config = readConfig()
    return new Listr([
      {
        title: 'Cloning Directory',
        task: async () => {
          if (!fs.existsSync(`${HOME_DIR}/credence`)) {
            fs.mkdirSync(`${path.resolve(`${HOME_DIR}/credence`)}`);
          }
          if (!fs.existsSync(directory)) {
            logger.info(`Since the given directory does not exist a new directory will be created at the path ${directory}`);
            fs.mkdirSync(`${directory}`);
          } else {
            logger.info(`cloning into ${directory}`);
          }
          await git.clone(`https://${gitInput.gitusername}:${gitInput.gitpassword}@gitlab.credenceanalytics.com/ideal-funds/idf-installations.git`, directory);
          config.init.isInstalled = true;
          config.init.installationMode = 'online';
          const newConfig = JSON.stringify(config, null, 2);
          fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
        }
      }
    ]);
  } catch (error) {
    logger.error(`Error in cloningDirectory due to : ${error}`);
    throw new CliError(`Error in cloningDirectory due to : ${error}`);
  }
}

async function extractionDirectory(directory) {
  try {
    logger.info('Starting Extraction Process');

    const config = readConfig()

    return new Listr([
      {
        title: 'Extracting Directory',
        task: () => {
          return new Listr([
            {
              title: 'Initializing',
              task: async () => {
                if (fs.existsSync(directory)) {
                  logger.info(`Directory: ${HOME_DIR}`);
                  if (!fs.existsSync(`${HOME_DIR}/credence`)) {
                    fs.mkdirSync(`${path.resolve(`${HOME_DIR}/credence`)}`);
                  }
                  if (!fs.existsSync((`${HOME_DIR}/credence/idf-installations.zip`))) {

                    if (!directory.includes(path.resolve(`credence`))) {
                      fs.copyFileSync(directory, path.resolve(`${HOME_DIR}/credence/idf-installations.zip`));
                      fs.rmSync(directory, { recursive: true, force: true });
                    } else {
                      // await execPromise(`sudo mv ${directory} ${HOME_DIR}/credence/idf-installations.zip`)
                      fs.renameSync(directory, `${HOME_DIR}/credence/idf-installations.zip`);
                    }
                  }
                } else {
                  throw new CliError('Provided directory path does not exist')
                }
              }
            },
            {
              title: 'Extracting',
              task: async () => {
                if (fs.existsSync(path.resolve(`${HOME_DIR}/credence/idf-installations.zip`))) {
                  await extract(path.resolve(`${HOME_DIR}/credence/idf-installations.zip`), { dir: `${HOME_DIR}/credence` });
                  if (fs.existsSync(`${HOME_DIR}/credence/idf-installations-master`)) {
                    fs.renameSync(path.resolve(`${HOME_DIR}/credence/idf-installations-master`), path.resolve(`${HOME_DIR}/credence/idf-installations`));
                  }
                  config.init.isInstalled = true;
                  config.init.installationMode = 'offline';
                  const newConfig = JSON.stringify(config, null, 2);
                  fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
                } else {
                  throw new CliError('Provided directory path does not exist')
                }
              }
            }
          ]);
        }
      }
    ]);
  } catch (error) {
    logger.error(`Error in extractionDirectory due to : ${error}`);
    throw new CliError(`Error in extractionDirectory due to : ${error}`);
  }
}

function initUninstall(initDetails) {
  try {
    const config = readConfig();
    return new Listr([
      {
        title: 'Deleting Installation Files',
        task: () => {
          if (initDetails.isInstalled) {
            fs.rmSync(initDetails.path, { recursive: true, force: true });
            config.init.path = '';
            config.init.isInstalled = false;
            config.init.installationMode = '';
            const newConfig = JSON.stringify(config, null, 2);
            fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
          } else {
            logger.info('idf-installations is not installed');
          }
        }
      }
    ]);
  } catch (error) {
    logger.error(`Error in initUninstall due to : ${error}`);
    throw new CliError(`Error in initUninstall due to : ${error}`);
  }
}

module.exports = { cloningDirectory, extractionDirectory, initUninstall };
