const path = require('path');
const os = require('os');
const fs = require('fs');
const { prompt } = require('enquirer');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));
const { errorHandler } = require(path.join(__dirname, '../../utils/utils.js'));
const { cloningDirectory, extractionDirectory } = require(path.join(__dirname, '../../utils/helper/init-helper.js'));

const logger = initLogger('product');

async function initInput(all) {
  try {
    let init;
    let directoryForInit = { path: '' };
    let gitInput;
    if (all === 'no') {
      init = await prompt([
        {
          type: 'select',
          name: 'answer',
          message: 'Would you like to download the installer files ? ',
          choices: ['yes', 'no'],
        }
      ]);
    }
    if (all === 'yes' || init.answer === 'yes') {
      const installationType = await prompt({
        type: 'select',
        name: 'answer',
        message: 'Pick Installation Mode',
        choices: ['online', 'offline'],
      });

      if (installationType.answer === 'offline') {
        directoryForInit = await prompt([
          {
            type: 'input',
            name: 'path',
            message: 'Please enter the path to the installer zip file',
            initial: path.resolve(`${HOME_DIR}/credence/idf-installations.zip`),
            result: (input) => {
              return path.resolve(input.trim());
            }
          }
        ]);
      } else {
        gitInput = await prompt([
          {
            type: 'input',
            name: 'gitusername',
            message: 'Please enter your gitlab username',
            initial: 'credence-user'
          },
          {
            type: 'password',
            name: 'gitpassword',
            message: 'Please enter your gitlab password',
            result: (password) => {
              return encodeURIComponent(password);
            }
          }
        ]);
      }
      return {
        installationType,
        directoryForInitPath: directoryForInit.path,
        gitInput,
        init,
      };
    }
    return { init };
  } catch (err) {
    errorHandler(err, 'product')
    throw new CliError(`Error occurred during taking init input : ${error.stack}`)
  }
}

async function initIwf(installationType, directory = '', gitInput = '') {
  try {
    const config = readConfig()
    config.init.path = `${HOME_DIR}/credence/idf-installations`;
    if (installationType.answer === 'online') {
      if (!fs.existsSync(config.init.path)) {
        return await cloningDirectory(config.init.path, gitInput);
      } else {
        logger.info('Installations folder is already downloaded locally');
      }
    } else {
      if (!fs.existsSync(path.resolve(config.init.path))) {
        return await extractionDirectory(directory);
      } else {
        logger.info('Installations folder is already downloaded locally');
      }
    }
    const newConfig = JSON.stringify(config, null, 2);
    fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
  } catch (error) {
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during initIwf : ${error.stack}`)
  }
}

module.exports = { initIwf, initInput };
