const fs = require('fs');
const path = require('path');
const os = require('os');
const { exec } = require('child_process');
const { readConfig } = require('../utils');

const execPromise = require('util').promisify(exec);

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');

async function oracleRedhat8_6Install(directory, folderName) {
	try {
		const config = readConfig();
		const oracleResult = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/oracle-instantclient.sh ${directory} ${HOME_DIR}`)
		if (oracleResult.stdout.length > 0) logger.info(`stdout : ${oracleResult.stdout}`);
		if (oracleResult.stderr.length > 0) logger.info(`stderr : ${oracleResult.stderr}`);
		if (oracleResult.stdout.includes('oracle-instantclient18.5-basic-18.5.0.0.0-3.x86_64')) {
			config.dependencies.oracleInstantClient.path = `${HOME_DIR}/credence/installation/oracleinstantclient`;
			config.dependencies.oracleInstantClient.isInstalled = true;
			const newConfig = JSON.stringify(config, null, 2);
			fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
		} else logger.error(`oracle-instantclient was not installed properly `);
	} catch (err) {
		throw new CliError(`Execution of oracle-instanctclient.sh failed due to ${err.stack}`)
	}
}
async function oracleRedhat9Install(directory, folderName) {
	try {
		const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/oracle-instantclient.sh ${directory} ${HOME_DIR}`)
		if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
		if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
	} catch (err) {
		throw new CliError(`Execution of oracle-instanctclient.sh failed due to ${err.stack}`)
	}
}
async function oracleUbuntu20Install(directory, folderName) {
	try {
		const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/oracle-instantclient.sh ${directory} ${HOME_DIR}`)
		if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
		if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
	} catch (err) {
		throw new CliError(`Execution of oracle-instanctclient.sh failed due to ${err.stack}`)
	}
}


async function oracleRedhat8_6Uninstall() {
	try {
		const config = readConfig();
		const oracleResult = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat8_6/scripts/uninstall/oracle-instantclient.sh ${HOME_DIR}`)
		if (oracleResult.stdout.length > 0) logger.info(`stdout : ${oracleResult.stdout}`);
		if (oracleResult.stderr.length > 0) logger.info(`stderr : ${oracleResult.stderr}`);
		config.dependencies.oracleInstantClient.path = '';
		config.dependencies.oracleInstantClient.isInstalled = false;
		const newConfig = JSON.stringify(config, null, 2);
		fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
	} catch (err) {
		throw new CliError(`Execution of oracle-instanctclient.sh failed due to ${err.stack}`)
	}
}
async function oracleRedhat9Uninstall(folderName) {
	try {
		const value = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/${folderName}/scripts/uninstall/oracle-instantclient.sh ${config.init.path} ${HOME_DIR}`)
		if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
		if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
	} catch (err) {
		throw new CliError(`Execution of oracle-instanctclient.sh failed due to ${err.stack}`)
	}
}
async function oracleUbuntu20Uninstall() {
	try {
		const value = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat8_6/oracle-instantclient.sh ${config.init.path} ${HOME_DIR}`)
		if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
		if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
	} catch (err) {
		throw new CliError(`Execution of oracle-instanctclient.sh failed due to ${err.stack}`)
	}
}


async function oracleInstallation(directory) {
	try {
		const config = readConfig();

		if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence/installation`))) {
			fs.mkdirSync(path.resolve(`${HOME_DIR}/credence/installation`));
		}
		switch (true) {
			case config.os.version.includes('8'):
				await oracleRedhat8_6Install(directory, 'redhat8_6');
				break;
			case config.os.version.includes('9'):
				await oracleRedhat9Install(directory, 'redhat9');
				break;
			case config.os.version.includes('ubuntu'):
				await oracleUbuntu20Install(directory, 'redhat8_6');
				break;
			default:
				logger.info('No matching case found.');
		}
	} catch (error) {
		logger.error(`Error in oracleInstallation due to : ${error}`);
		throw new CliError(`Error in oracleInstallation due to : ${error}`);
	}
}

async function oracleUninstall(oracleDetails) {
	try {
		const config = readConfig()
		if (oracleDetails.isInstalled) {
			switch (true) {
				case config.os.version.includes('8'):
					await oracleRedhat8_6Uninstall();
					break;
				case config.os.version.includes('9'):
					await oracleRedhat9Uninstall('redhat9');
					break;
				case config.os.version.includes('ubuntu'):
					await oracleUbuntu20Uninstall();
					break;
			}
		} else {
			logger.info('instantclient is not installed');
		}
	} catch (error) {
		logger.error(`Error in oracleUninstall due to : ${error}`);
		throw new CliError(`Error in oracleUninstall due to : ${error}`);
	}
}

module.exports = { oracleInstallation, oracleUninstall };
