const extract = require('extract-zip');
const fs = require('fs');
const os = require('os');
const path = require('path');
const xml2js = require('xml2js');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');




async function iwfWarInstallation(directory) {
  try {
    const config = readConfig()

    if (config.services.applicationServer.isInstalled) {
      if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence`))) {
        fs.mkdirSync(`${HOME_DIR}/credence`);
      }

      if (config.services.applicationServer.type === 'wildfly') {
        fs.copyFileSync(`${directory}/Microservices/Application-Server/FUNDS_RELEASE.war.zip`, `${HOME_DIR}/credence/wildfly/standalone/deployments/FUNDS_RELEASE.war.zip`)
        await extract(`${HOME_DIR}/credence/wildfly/standalone/deployments/FUNDS_RELEASE.war.zip`, { dir: `${HOME_DIR}/credence/wildfly/standalone/deployments/` });
        fs.rmSync(path.resolve(`${HOME_DIR}/credence/wildfly/standalone/deployments/FUNDS_RELEASE.war.zip`), { force: true });
        const xml = fs.readFileSync(`${HOME_DIR}/credence/wildfly/standalone/deployments/FUNDS_RELEASE.war/WEB-INF/web.xml`, 'utf-8');
        xml2js.parseString(xml, (err, result) => {
          if (err) {
            throw err;
          }
          //server-path
          result['web-app'].servlet[0]['init-param'][0]['param-value'][0] = `${HOME_DIR}/credence/wildfly/standalone/deployments/FUNDS_RELEASE.war`;
          //jndiname
          result['web-app'].servlet[0]['init-param'][1]['param-value'][0] = '/funds_release';
          //config-file
          // result['web-app'].servlet[1]['init-param'][0]['param-value'][0] = 'funds_release.properties';
          const builder = new xml2js.Builder();
          const newXml = builder.buildObject(result);

          fs.writeFileSync(`${HOME_DIR}/credence/wildfly/standalone/deployments/FUNDS_RELEASE.war/WEB-INF/web.xml`, newXml);
        });
        config.services.applicationServer.warfile.path = `${HOME_DIR}/credence/wildfly/standalone/deployments/FUNDS_RELEASE.war`;
        config.services.applicationServer.warfile.isInstalled = true;
        config.services.applicationServer.warfile.type = 'FUNDS_RELEASE';
        const newConfig = JSON.stringify(config, null, 2);
        fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
      } else {
        fs.copyFileSync(`${path.resolve(`${directory}/Microservices/Application-Server/FUNDS_RELEASE.war.zip`)}`, `${path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/deployments/FUNDS_RELEASE.war.zip`)}`);
        await extract(`${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/deployments/FUNDS_RELEASE.war.zip`, { dir: `${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/deployments` });
        fs.rmSync(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/deployments/FUNDS_RELEASE.war.zip`), { force: true });
        const xml = fs.readFileSync(`${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/deployments/FUNDS_RELEASE.war/WEB-INF/web.xml`, 'utf-8');
        xml2js.parseString(xml, (err, result) => {
          if (err) {
            throw err;
          }
          result['web-app'].servlet[0]['init-param'][0]['param-value'][0] = `${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/deployments/FUNDS_RELEASE.war`;
          result['web-app'].servlet[0]['init-param'][1]['param-value'][0] = '/funds_release';
          result['web-app'].servlet[1]['init-param'][0]['param-value'][0] = 'funds_release.properties';
          result['web-app']['context-param'][1]['param-value'] = '/JBOSS-7.1PEMI/modules/org/quartz/main';
          const builder = new xml2js.Builder();
          const newXml = builder.buildObject(result);

          fs.writeFileSync(`${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/deployments/FUNDS_RELEASE.war/WEB-INF/web.xml`, newXml);
        });

        config.services.applicationServer.warfile.path = path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/deployments/FUNDS_RELEASE.war`);
        config.services.applicationServer.warfile.isInstalled = true;
        config.services.applicationServer.warfile.type = 'FUNDS_RELEASE';
        const newConfig = JSON.stringify(config, null, 2);
        fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
      }
    } else {
      logger.info('Application Server is not installed yet');
    }
  } catch (error) {
    logger.error(`Error in iwfWarInstallation due to : ${error}`);
    throw new CliError(`Error in iwfWarInstallation due to : ${error}`);
  }
}
function iwfWarUninstall(warfileDetails) {
  try {
    const config = readConfig()

    if (warfileDetails.isInstalled) {
      fs.rmSync(warfileDetails.path, { recursive: true, force: true });
      config.services.applicationServer.warfile.path = '';
      config.services.applicationServer.warfile.isInstalled = false;
      config.services.applicationServer.warfile.type = '';
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else {
      logger.info('iwf war is not installed');
    }
  } catch (error) {
    logger.error(`Error in iwfWarUninstall due to : ${error}`);
    throw new CliError(`Error in iwfWarUninstall due to : ${error}`);
  }
}
module.exports = { iwfWarInstallation, iwfWarUninstall };
