const fs = require('fs');

const path = require('path');
const util = require('util')
const execPromise = util.promisify(require('child_process').exec);
const os = require('os');
const chalk = require('chalk');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');


async function javaRedhat8_6Install(directory, folderName) {
  try {
    const config = readConfig();
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/java.sh ${directory}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`);
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`);
    if (value.stdout.includes(`java version "1.8.0_281"`) || value.stderr.includes(`java version "1.8.0_281"`)) {
      config.dependencies.java.path = '/usr/java';
      config.dependencies.java.isInstalled = true;
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else {
      logger.error(`Java was not installed properly`);
      throw new Error(`Java was not installed properly`);
    }
  } catch (err) {
    throw new CliError(`Execution of java.sh failed due to ${err.stack}`)
  }
}
async function javaRedhat9Install(directory, folderName) {
  try {
    logger.info(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/java.sh ${directory}`);
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/java.sh ${directory}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    logger.error(`Execution of java.sh failed due to ${err.stack}`);
    throw new CliError(`Execution of java.sh failed due to ${err.stack}`)
  }
}
async function javaUbuntu20Install(directory, folderName) {
  try {
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/java.sh ${directory}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of java.sh failed due to ${err.stack}`)
  }
}

async function javaRedhat8_6Uninstall() {
  try {
    const config = readConfig();
    const value = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat8_6/scripts/uninstall/java.sh`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`);
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`);
  } catch (err) {
    const config = readConfig();
    if (err.message.includes('java: command not found')) {
      config.dependencies.java.path = '';
      config.dependencies.java.isInstalled = false;
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else {
      throw new CliError(`Execution of java.sh failed due to ${err.stack}`)
    }
  }
}
async function javaRedhat9Uninstall(folderName) {
  try {
    const config = readConfig();
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/uninstall/java.sh ${javaDetails.path}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`);
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`);
    config.dependencies.java.path = '/usr/java';
    config.dependencies.java.isInstalled = true;
    const newConfig = JSON.stringify(config, null, 2);
    fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
  } catch (err) {
    throw new CliError(`Execution of java.sh failed due to ${err.stack}`)
  }
} async function javaUbuntu20Uninstall() {
  try {
    const config = readConfig();
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat8_6/scripts/uninstall/java.sh ${javaDetails.path}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`);
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`);
    config.dependencies.java.path = '/usr/java';
    config.dependencies.java.isInstalled = true;
    const newConfig = JSON.stringify(config, null, 2);
    fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
  } catch (err) {
    throw new CliError(`Execution of java.sh failed due to ${err.stack}`)
  }
}

async function javaInstallation(directory) {
  try {
    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence`))) {
      fs.mkdirSync(path.resolve(`${HOME_DIR}/credence`));
    }
    logger.info(`Starting Java Installation...`);
    const config = readConfig();
    switch (true) {
      case config.os.version.includes('8'):
        await javaRedhat8_6Install(directory, 'redhat8_6');
        break;
      case config.os.version.includes('9'):
        await javaRedhat9Install(directory, 'redhat9');
        break;
      case config.os.version.includes('ubuntu'):
        await javaUbuntu20Install(directory, 'redhat8_6');
        break;
      default:
        logger.info('No matching case found.');
    }

  } catch (error) {
    throw new CliError(`Error in javaInstallation due to : ${error}`);
  }
}

async function javaUninstall(javaDetails) {
  try {
    if (javaDetails.isInstalled) {
      const config = readConfig()
      switch (true) {
        case config.os.version.includes('8'):
          await javaRedhat8_6Uninstall();
          break;
        case config.os.version.includes('9'):
          await javaRedhat9Uninstall('redhat9');
          break;
        case config.os.version.includes('ubuntu'):
          await javaUbuntu20Uninstall();
          break;
      }
    } else {
      logger.info(chalk.redBright('java is not installed'));
    }
  } catch (e) {
    throw new CliError(`Uninstalling java : ${e.stack}`)
  }
}

module.exports = { javaInstallation, javaUninstall };
