const extract = require('extract-zip');
const fs = require('fs');
const { exec } = require('child_process');
const os = require('os');
const path = require('path');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');

async function jbossInstallation(directory) {
  try {
    const config = readConfig()
    if (!fs.existsSync(path.resolve(`${directory}/Microservices/Application-Server`))) {
      logger.error('Provided path for installation files is incorrect');
      return;
    }
    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence`))) {
      fs.mkdirSync(`${HOME_DIR}/credence`);
    }

    // await execPromise(`sudo cp  ${directory}/Microservices/Application-Server/JBOSS-7.1PEMI-17-March-2020.zip ${HOME_DIR}/credence/`);
    fs.copyFileSync(`${directory}/Microservices/Application-Server/JBOSS-7.1PEMI-17-March-2020.zip`, `${HOME_DIR}/credence/JBOSS-7.1PEMI-17-March-2020.zip`);

    await extract(`${HOME_DIR}/credence/JBOSS-7.1PEMI-17-March-2020.zip`, { dir: `${HOME_DIR}/credence` });
    fs.rmSync(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI-17-March-2020.zip`), { force: true });

    if (fs.existsSync(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI`))) {
      fs.rmSync(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/pemi_int.cfg`), { recursive: true, force: true });
      fs.rmSync(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/std_cloud_release_migration.cfg`), { recursive: true, force: true });
      fs.rmSync(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/pemi_int.properties`), { recursive: true, force: true });
      fs.rmSync(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/std_cloud_release_migration.properties`), { recursive: true, force: true });

      fs.renameSync(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/funds_release.cfg`), path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/funds_release.cfg`));
      fs.renameSync(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/funds_release.properties`), path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/funds_release.properties`));

      fs.readFile(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/funds_release.cfg`), (error, datam) => {
        if (error) {
          logger.error(`error while reading file : ${error}`);
        }

        let data = datam.toString();

        const newData = data
          .replace(/orgMsgLogPath=.*/, `orgMsgLogPath=${path.join(HOME_DIR, 'credence', 'JBOSS-7.1PEMI', 'standalone', 'log')}`)
          .replace(/serverPath=.*/, `serverPath=${path.join(HOME_DIR, 'credence', 'JBOSS-7.1PEMI', 'standalone', 'deployments', 'FUNDS_RELEASE.war')}`)
          .replace(/jndiName=.*/, 'jndiName=/funds_release');

        fs.writeFile(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/funds_release.cfg`, newData, (err) => {
          if (err) {
            logger.error(`error while writing file : ${err}`);
          }
        });
      });

      fs.readFile(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/funds_release.properties`), (error, datam) => {
        if (error) {
          logger.error(`error while reading file : ${error}`);
        }

        let data = datam.toString();
        const newData = data
          .replace(/org.quartz.dataSource.funds_release.jndiURL =.*/, 'org.quartz.dataSource.funds_release.jndiURL = java:/funds_release')
          .replace(/org.quartz.jobStore.dataSource =.*/, `org.quartz.jobStore.dataSource = funds_release`)
          .replace(/org.quartz.scheduler.instanceName =.*/, 'org.quartz.scheduler.instanceName = funds_release');

        fs.writeFile(`${HOME_DIR}/credence/JBOSS-7.1PEMI/modules/org/quartz/main/funds_release.properties`, newData, (err) => {
          if (err) {
            logger.error(`error while writing file : ${err}`);
          }
        });
      });

      fs.readFile(path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/configuration/standalone.xml`), (error, datam) => {
        if (error) {
          logger.error(`error while reading file : ${error}`);
        }

        let data = datam.toString();
        const newData = data.replace('datasource jta="false" jndi-name="java:/funds_release" pool-name="funds_release', 'datasource jta="false" jndi-name="funds_release" pool-name="funds_release');

        fs.writeFile(`${HOME_DIR}/credence/JBOSS-7.1PEMI/standalone/configuration/standalone.xml`, newData, (err) => {
          if (err) {
            logger.error(`error while writing file : ${err}`);
          }
        });
      });
      config.services.applicationServer.path = path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI`);
      config.services.applicationServer.isInstalled = true;
      config.services.applicationServer.type = 'jboss';
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else {
      logger.error('jboss was not installed properly');
    }
  } catch (error) {
    logger.error(`Error in jbossInstallation due to : ${error}`);
    throw new CliError(`Error in jbossInstallation due to : ${error}`);
  }
}
async function jbossUninstall(jbossDetails) {
  try {
    if (jbossDetails.isInstalled) {
      const config = readConfig();
      // await execPromise(`sudo rm -rf ${jbossDetails.path}`);
      fs.rmSync(jbossDetails.path, { recursive: true, force: true })
      config.services.applicationServer.path = '';
      config.services.applicationServer.isInstalled = false;
      config.services.applicationServer.type = '';
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else {
      logger.info('jboss is not installed');
    }
  } catch (error) {
    logger.error(`Error in jbossUninstall due to : ${error}`);
    throw new CliError(`Error in jbossUninstall due to : ${error}`);
  }
}

module.exports = { jbossInstallation, jbossUninstall };
