const extract = require('extract-zip');
const fs = require('fs');
const { exec } = require('child_process');

const os = require('os');
const path = require('path');
const Listr = require('listr');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');

async function mailingEngineInstallation(directory) {
  try {
    const config = readConfig()

    if (!fs.existsSync(path.resolve(`${directory}/Mailing-Engine`))) {
      logger.error('Provided path for installation files is incorrect');
      return;
    }
    fs.copyFileSync(`${directory}/Mailing-Engine/mailing-engine.zip`, `${HOME_DIR}/credence/mailing-engine.zip`)

    await extract(`${HOME_DIR}/credence/mailing-engine.zip`, { dir: `${HOME_DIR}/credence` });
    fs.rmSync(path.resolve(`${HOME_DIR}/credence/mailing-engine.zip`), { force: true });
    if (fs.existsSync(path.resolve(`${HOME_DIR}/credence/mailing-engine/`))) {
      config.mailingEngine.path = path.resolve(`${HOME_DIR}/credence/mailing-engine`);
      config.mailingEngine.isInstalled = true;
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else {
      logger.error('mailing-engine was not installed properly');
    }
  } catch (error) {
    logger.error(`Error in mailingEngineInstallation due to : ${error}`);
    throw new CliError(`Error in mailingEngineInstallation due to : ${error}`);
  }
}

function mailingEngineUninstall(mailingEngineDetails) {
  try {
    const config = readConfig()

    return new Listr([
      {
        title: 'Deleting Mailing Engine',
        task: () => {
          if (mailingEngineDetails.isInstalled) {
            fs.rmSync(mailingEngineDetails.path, { recursive: true, force: true });
            config.mailingEngine.path = '';
            config.mailingEngine.isInstalled = false;
            const newConfig = JSON.stringify(config, null, 2);
            fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
          } else {
            logger.info('mailing engine is not installed');
          }
        }
      }
    ]);
  } catch (error) {
    logger.error(`Error in mailingEngineUninstall due to : ${error}`);
    throw new CliError(`Error in mailingEngineUninstall due to : ${error}`);
  }
}

module.exports = { mailingEngineInstallation, mailingEngineUninstall };
