const path = require('path');
const os = require('os');
const fs = require('fs');
const { prompt } = require('enquirer');
const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));
const { errorHandler } = require(path.join(__dirname, '../../utils/utils.js'));
const { mailingEngineInstallation } = require(path.join(__dirname, '../../utils/helper/mailing-engine-helper.js'));

const logger = initLogger('product');

async function mailingEngineInput(all) {
  try {
    if (all === 'no') {
      const mailInput = await prompt([
        {
          type: 'select',
          name: 'answer',
          message: 'Would you like to install mailing engine ? ',
          choices: ['yes', 'no'],
        }
      ]);
      return mailInput.answer;
    }
  } catch (error) {
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during taking mailing input : ${error.stack}`)

  }
}

async function mailingEngine(directory) {
  try {
    if (fs.existsSync(path.resolve(`${HOME_DIR}/credence/mailing-engine`))) {
      logger.info('Mailing Engine is already installed ');
    } else {
      await mailingEngineInstallation(directory);
      logger.info('Mailing Engine is installed successfully');
    }
  } catch (error) {
    logger.error(`Error occurred : ${error}`);
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during mailingEngine : ${error.stack}`)

  }
}

module.exports = { mailingEngine, mailingEngineInput };
