const extract = require('extract-zip');
const fs = require('fs');
const os = require('os');
const path = require('path');
const Listr = require('listr');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));
const { appServer } = require(path.join(__dirname, './app-server.js'));

const logger = initLogger('product');

async function microservicesInstallation(directory, all, selectedFiles = [], appServerChoice) {
  try {
    const config = readConfig()

    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence/microservices`))) {
      fs.mkdirSync(`${HOME_DIR}/credence/microservices`);
    }
    if (all === 'yes') {
      try {
        const files = (await fs.promises.readdir(`${directory}/Microservices/`))
          .filter(file => file.includes('.zip'))
        files.forEach(file => {
          fs.copyFileSync(`${directory}/Microservices/${file}`, `${HOME_DIR}/credence/microservices/${file}`)
        })
      } catch (e) {
        throw new CliError(`Error during copy : ${e.stack}`)
      }

      const files = fs.readdirSync(path.resolve(`${HOME_DIR}/credence/microservices`)).filter((el) => el.includes('.zip'));
      if (files.length > 0) {
        const tasks = new Listr([]);
        for (const file of files) {
          tasks.add({
            title: `${file.slice(0, -4).toUpperCase()}`,
            task: async () => {
              if (!config.services[file.slice(0, -4)].isInstalled) {
                await extract(`${HOME_DIR}/credence/microservices/${file}`, { dir: `${HOME_DIR}/credence/microservices` });
                fs.rmSync(`${HOME_DIR}/credence/microservices/${file}`, { force: true });
                logger.info(`${file} installed`);
                config.services[file.slice(0, -4)].path = `${HOME_DIR}/credence/microservices/${file.slice(0, -4)}`;
                config.services[file.slice(0, -4)].isInstalled = true;
                const newConfig = JSON.stringify(config, null, 2);
                fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
              }
            }
          });
        }
        if (appServerChoice.appServerInput.answer === 'yes') {
          tasks.add({
            title: `${appServerChoice.appServerValues.answer}`,
            task: async () => await appServer(directory, appServerChoice.appServerValues),
          });
        }
        return tasks;
      } else {
        throw new CliError(`Empty files object : ${files} `)
      }
    } else {
      const tasks = new Listr([]);
      for (const file of selectedFiles) {
        fs.copyFileSync(`${directory}/Microservices/${file}`, `${HOME_DIR}/credence/microservices/${file}`);
        tasks.add({
          title: `${file.slice(0, -4).toUpperCase()}`,
          task: async () => {
            if (config.services[file.slice(0, -4)].isInstalled === false) {
              await extract(`${HOME_DIR}/credence/microservices/${file}`, { dir: `${HOME_DIR}/credence/microservices/` });
              logger.info(`${file} installed`);
              fs.rmSync(`${HOME_DIR}/credence/microservices/${file}`, { force: true });
              config.services[file.slice(0, -4)].path = `${HOME_DIR}/credence/microservices/${file.slice(0, -4)}`;
              config.services[file.slice(0, -4)].isInstalled = true;
              const newConfig = JSON.stringify(config, null, 2);
              fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
            }
          }
        });
      }
      if (appServerChoice.appServerInput.answer === 'yes') {
        tasks.add({
          title: `Installing ${appServerChoice.appServerValues.answer}`,
          task: async () => await appServer(directory, appServerChoice.appServerValues),
        });
      }
      return tasks;
    }
  } catch (error) {
    logger.error(`Error in microservicesInstallation due to : ${error}`);
    throw new CliError(`Error in microservicesInstallation due to : ${error}`);
  }
}

function microservicesUninstall(servicesDetails) {
  try {
    const config = readConfig()


    const services = Object.keys(servicesDetails);
    for (const service of services) {
      if (servicesDetails[service].isInstalled) {
        fs.rmSync(servicesDetails[service].path, { recursive: true, force: true });
        config.services[service].path = ``;
        config.services[service].isInstalled = false;
      } else {
        logger.info(`${service} is not installed`);
      }
    }
    fs.rmSync(`${HOME_DIR}/credence/microservices`, { recursive: true, force: true });
    const newConfig = JSON.stringify(config, null, 2);
    fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
  } catch (error) {
    logger.error(`Error in microservicesUninstall due to : ${error}`);
    throw new CliError(`Error in microservicesUninstall due to : ${error}`);
  }
}

module.exports = { microservicesInstallation, microservicesUninstall };
