const fs = require('fs');
const path = require('path');
const os = require('os');
const { exec } = require('child_process');
const execPromise = require('util').promisify(exec);
const mysql = require('mysql2/promise');
const { prompt } = require('enquirer');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');




async function mysqlRedhat8_6Install(directory, folderName) {
  try {
    const config = readConfig();
    const mysqlResult = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/mysql.sh ${directory} ${HOME_DIR}`)
    if (mysqlResult.stdout.length > 0) logger.info(`stdout : ${mysqlResult.stdout}`);
    if (mysqlResult.stderr.length > 0) logger.info(`stderr : ${mysqlResult.stderr}`);
    if (mysqlResult.stdout.includes('mysqld.service - MySQL 8.0 database server') && mysqlResult.stdout.includes('Active: active (running)') && mysqlResult.stdout.includes('mysql  Ver 8.0.26 for Linux on x86_64 (Source distribution)')) {
      config.dependencies.MySql.path = `${HOME_DIR}/credence/installation/mysql`;
      config.dependencies.MySql.isInstalled = true;
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else logger.error(`mysql was not installed properly `);
  } catch (err) {
    throw new CliError(`Execution of mysql.sh failed due to ${err.stack}`)
  }
}
async function mysqlRedhat9Install(directory, folderName) {
  try {
    const config = readConfig();
    logger.info(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/install/mysql.sh ${directory}`);
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/install/mysql.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of mysql.sh failed due to ${err.stack}`)
  }
}
async function mysqlUbuntu20Install(directory, folderName) {
  try {

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/ubuntu20/scripts/install/mysql.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of mysql.sh failed due to ${err.stack}`)
  }
}


async function mysqlRedhat8_6Uninstall() {
  try {
    const config = readConfig();
    const mysqlResult = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat8_6/scripts/uninstall/mysql.sh ${HOME_DIR}`)
    if (mysqlResult.stdout.length > 0) logger.info(`stdout : ${mysqlResult.stdout}`);
    if (mysqlResult.stderr.length > 0) logger.info(`stderr : ${mysqlResult.stderr}`);
    fs.readFile(path.resolve(`${HOME_DIR}/.bashrc`), (error, datam) => {
      if (error) {
        logger.error(`error while reading file : ${error}`);
      }
      let data = datam.toString();
      const newData = data.replace(/password=.*/, `password="`)
      fs.writeFile(`${HOME_DIR}/.bashrc`, newData, (err) => {
        if (err) {
          logger.error(`Error during writing .bashrc : ${err.stack}`)
        }
      });
    });
    config.dependencies.MySql.path = ``;
    config.dependencies.MySql.isInstalled = false;
    const newConfig = JSON.stringify(config, null, 2);
    fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
  } catch (err) {
    throw new CliError(`Execution of mysql.sh failed due to ${err.stack}`)
  }
}
async function mysqlRedhat9Uninstall() {
  try {
    const config = readConfig();
    const value = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat9/scripts/uninstall/mysql.sh ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of mysql.sh failed due to ${err.stack}`)
  }
}
async function mysqlUbuntu20Uninstall() {
  try {

    const value = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/ubuntu20/scripts/uninstall/mysql.sh ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of mysql.sh failed due to ${err.stack}`)
  }
}

async function mysqlInstallation(directory) {
  try {
    const config = readConfig()
    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence/installation`))) {
      fs.mkdirSync(path.resolve(`${HOME_DIR}/credence/installation`));
    }
    switch (true) {
			case config.os.version.includes('8'):
				await mysqlRedhat8_6Install(directory, 'redhat8_6');
				break;
			case config.os.version.includes('9'):
				await mysqlRedhat9Install(directory, 'redhat9');
				break;
			case config.os.version.includes('ubuntu'):
				await mysqlUbuntu20Install(directory, 'redhat8_6');
				break;
			default:
				logger.info('No matching case found.');
		}

  } catch (error) {
    logger.error(`Error in mysqlInstallation due to : ${error}`);
    throw new CliError(`Error in mysqlInstallation due to : ${error}`);
  }
}



async function mysqlUninstall(mysqlDetails) {
  try {
    if (mysqlDetails.isInstalled) {
      const config = readConfig()

      switch (true) {
        case config.os.version.includes('8'):
          await mysqlRedhat8_6Uninstall();
          break;
        case config.os.version.includes('9'):
          await mysqlRedhat9Uninstall();
          break;
        case config.os.version.includes('ubuntu'):
          await mysqlUbuntu20Uninstall();
          break;
      }

    } else {
      logger.info('Mysql is not installed');
    }
  } catch (error) {
    logger.error(`Error in mysqlUninstall due to : ${error}`);
    throw new CliError(`Error in mysqlUninstall due to : ${error}`);
  }
}

async function mysqlConfig() {
  try {
    const connection = await mysql.createConnection({
      host: 'localhost',
      user: 'root',
      password: '',
      database: 'mysql',
    });

    const setNewPassword = await prompt([
      {
        type: 'select',
        name: 'answer',
        message: 'Would you like to set the password for MySql root user',
        choices: ['yes', 'no'],
      }
    ]);

    if (setNewPassword.answer === 'yes') {
      const mysqlPassword = await prompt([
        {
          type: 'password',
          name: 'answer',
          message: 'Enter the password you want to set for MySql server',
          validate: (password) => {
            if (password.length >= 8 && /[A-Z]/.test(password) && /[a-z]/.test(password) && /[!@#\$%\^\&*\)\(+=._-]/.test(password)) {
              return true;
            } else {
              return 'Password must be at least 8 characters long and contain at least 1 capital letter, 1 small letter and 1 special symbol.';
            }
          },
          result: (password) => {
            return password;
          }
        }
      ]);

      await connection.query(`ALTER USER 'root'@'localhost' IDENTIFIED BY '${mysqlPassword.answer}';`);
      await connection.query(`DROP DATABASE IF EXISTS test;`);
      await connection.query(`FLUSH PRIVILEGES;`);
      fs.readFile(path.resolve(`${HOME_DIR}/.bashrc`), (error, datam) => {
        if (error) {
          logger.error(`error while reading file : ${error}`);
        }
        let data = datam.toString();
        const newData = data.replace(/password=.*/, `password=${mysqlPassword.answer}"`)
        fs.writeFile(`${HOME_DIR}/.bashrc`, newData, (err) => {
          if (err) {
            logger.error(`error while writing file : ${err}`);
          }
        });
      });
    }
    connection.end();
  } catch (error) {
    if (error.message.includes(`Access denied for user 'root'@'localhost'`)) {
      logger.info('Mysql password has already been set')
    } else if (error.message.includes('connect ECONNREFUSED 127.0.0.1:3306')) {
      logger.info('Mysql is not installed');
    } else {
      logger.error(`Error occurred due to ${error.message}`)
      throw new CliError(`Error occured while setting mysql password : ${error.stack ? error.stack : error}`)
    }
  }
}

module.exports = { mysqlInstallation, mysqlUninstall, mysqlConfig };
