const fs = require('fs');
const path = require('path');
const { exec } = require('child_process');
const execPromise = require('util').promisify(exec);
const os = require('os');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');


async function nginxRedhat8_6Install(directory, folderName) {
  try {
    const config = readConfig();
    const nginxResult = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/nginx.sh ${directory} ${HOME_DIR}`)
    if (nginxResult.stdout.length > 0) logger.info(`stdout : ${nginxResult.stdout}`);
    if (nginxResult.stderr.length > 0) logger.info(`stderr : ${nginxResult.stderr}`);
    if (nginxResult.stdout.includes('nginx.service - nginx - high performance web server') && nginxResult.stdout.includes('Active: active (running)')) {
      config.dependencies.nginx.path = `/etc/nginx`;
      config.dependencies.nginx.isInstalled = true;
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else logger.error(`nginx was not installed properly `);
  } catch (err) {
    throw new CliError(`Execution of nginx.sh failed due to ${err.stack}`)
  }
}
async function nginxRedhat9Install(directory, folderName) {
  try {
    const config = readConfig();

    logger.info(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/nginx.sh ${directory} ${HOME_DIR}`);
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/nginx.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of nginx.sh failed due to ${err.stack}`)
  }
}
async function nginxUbuntu20Install(directory, folderName) {
  try {

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/uninstall/nginx.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of nginx.sh failed due to ${err.stack}`)
  }
}

async function nginxRedhat8_6Uninstall() {
  try {
    const config = readConfig();
    const nginxResult = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat8_6/scripts/uninstall/nginx.sh`)
    if (nginxResult.stdout.length > 0) logger.info(`stdout : ${nginxResult.stdout}`);
    if (nginxResult.stderr.length > 0) logger.info(`stderr : ${nginxResult.stderr}`);
    config.dependencies.nginx.path = ``;
    config.dependencies.nginx.isInstalled = false;
    const newConfig = JSON.stringify(config, null, 2);
    fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
  } catch (err) {
    throw new CliError(`Execution of nginx.sh failed due to ${err.stack}`)
  }
}
async function nginxRedhat9Uninstall() {
  try {
    const config = readConfig();
    logger.info(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/uninstall/nginx.sh ${directory}`);
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/uninstall/nginx.sh ${directory}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of nginx.sh failed due to ${err.stack}`)
  }
}
async function nginxUbuntu20Uninstall() {
  try {

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/ubuntu20/scripts/uninstall/nginx.sh ${directory}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of nginx.sh failed due to ${err.stack}`)
  }
}

async function nginxInstallation(directory) {
  try {
    const config = readConfig()

    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence/installation`))) {
      fs.mkdirSync(path.resolve(`${HOME_DIR}/credence/installation`));
    }
    switch (true) {
      case config.os.version.includes('8'):
        await nginxRedhat8_6Install(directory, 'redhat8_6');
        break;
      case config.os.version.includes('9'):
        await nginxRedhat9Install(directory, 'redhat9');
        break;
      case config.os.version.includes('ubuntu'):
        await nginxUbuntu20Install(directory, 'redhat8_6');
        break;
      default:
        logger.info('No matching case found.');
    }

  } catch (error) {
    logger.error(`Error in nginx installation due to : ${error.stack}`);
    throw new CliError(`Error in nginx installation due to : ${error.stack}`);
  }
}

async function nginxUninstall(nginxDetails) {
  try {
    if (nginxDetails.isInstalled) {
      const config = readConfig()

      switch (true) {
        case config.os.version.includes('8'):
          await nginxRedhat8_6Uninstall();
          break;
        case config.os.version.includes('9'):
          await nginxRedhat9Uninstall();
          break;
        case config.os.version.includes('ubuntu'):
          await nginxUbuntu20Uninstall();
          break;
      }
    } else {
      logger.info('nginx is not installed');
    }
  } catch (error) {
    logger.error(`Error in nginxUninstall due to : ${error}`);
    throw new CliError(`Error in nginxUninstall due to : ${error}`);
  }
}

module.exports = { nginxInstallation, nginxUninstall };
