const fs = require('fs');
const path = require('path');
const os = require('os');
const { readConfig } = require('../utils');
const execPromise = require('util').promisify(require('child_process').exec);

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');



async function pythonRedhat8_6Install(directory, folderName) {
  try {
    const config = readConfig();
    const pythonResult = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat8_6/scripts/install/python.sh ${directory} ${HOME_DIR}`)
    if (pythonResult.stdout.length > 0) logger.info(`stdout : ${pythonResult.stdout}`);
    if (pythonResult.stderr.length > 0) logger.info(`stderr : ${pythonResult.stderr}`);
    if (pythonResult.stdout.includes('python3 is /usr/bin/python3') && pythonResult.stdout.includes('Python 3.6.8')) {
      config.dependencies.python.path = `${HOME_DIR}/credence/installation/python`;
      config.dependencies.python.isInstalled = true;
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else logger.error(`python was not installed properly `);
  } catch (err) {
    throw new CliError(`Execution of python.sh failed due to ${err.stack}`)
  }
}
async function pythonRedhat9Install(directory, folderName) {
  try {
    const config = readConfig();

    logger.info(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/python.sh ${directory} ${HOME_DIR}`);
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/install/python.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of python.sh failed due to ${err.stack}`)
  }
}
async function pythonUbuntu20Install(directory, folderName) {
  try {

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/ubuntu20/scripts/install/python.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of python.sh failed due to ${err.stack}`)
  }
}

async function pythonRedhat8_6Uninstall() {
  try {
    const config = readConfig();
    const pythonResult = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat8_6/scripts/uninstall/python.sh ${HOME_DIR}`)
    if (pythonResult.stdout.length > 0) logger.info(`stdout : ${pythonResult.stdout}`);
    if (pythonResult.stderr.length > 0) logger.info(`stderr : ${pythonResult.stderr}`);
    config.dependencies.python.path = ``;
    config.dependencies.python.isInstalled = false;
    const newConfig = JSON.stringify(config, null, 2);
    fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);

  } catch (err) {
    throw new CliError(`Execution of python.sh failed due to ${err.stack}`)
  }
}
async function pythonRedhat9Uninstall() {
  try {
    const config = readConfig();

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/uninstall/python.sh ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of python.sh failed due to ${err.stack}`)
  }
}
async function pythonUbuntu20Uninstall() {
  try {

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/ubuntu20/scripts/uninstall/python.sh ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of python.sh failed due to ${err.stack}`)
  }
}

async function pythonInstallation(directory) {
  try {
    const config = readConfig();
    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence/installation`))) {
      fs.mkdirSync(path.resolve(`${HOME_DIR}/credence/installation`));
    }
    switch (true) {
			case config.os.version.includes('8'):
				await pythonRedhat8_6Install(directory, 'redhat8_6');
				break;
			case config.os.version.includes('9'):
				await pythonRedhat9Install(directory, 'redhat9');
				break;
			case config.os.version.includes('ubuntu'):
				await pythonUbuntu20Install(directory, 'redhat8_6');
				break;
			default:
				logger.info('No matching case found.');
		}
  } catch (error) {
    logger.error(`Error in pythonInstallation due to : ${error}`);
    throw new CliError(`Error in pythonInstallation due to : ${error}`);
  }
}

async function pythonUninstall(pythonDetails) {
  try {
    if (pythonDetails.isInstalled) {
      const config = readConfig();

      switch (true) {
        case config.os.version.includes('8'):
          await pythonRedhat8_6Uninstall();
          break;
        case config.os.version.includes('9'):
          await pythonRedhat9Uninstall();
          break;
        case config.os.version.includes('ubuntu'):
          await pythonUbuntu20Uninstall();
          break;
      }
    } else {
      logger.info('python is not installed');
    }
  } catch (error) {
    logger.error(`Error in pythonUninstall due to : ${error}`);
    throw new CliError(`Error in pythonUninstall due to : ${error}`);
  }
}

module.exports = { pythonInstallation, pythonUninstall };
