const fs = require('fs');
const path = require('path');
const execPromise = require('util').promisify(require('child_process').exec);
const os = require('os');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');



async function rabbitmqRedhat8_6Install(directory) {
  try {
    const config = readConfig();
    const rabbitmqResult = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat8_6/scripts/install/rabbitmq.sh ${directory} ${HOME_DIR}`)
    if (rabbitmqResult.stdout.length > 0) logger.info(`stdout : ${rabbitmqResult.stdout}`);
    if (rabbitmqResult.stderr.length > 0) logger.info(`stderr : ${rabbitmqResult.stderr}`);
    if (rabbitmqResult.stdout.includes('rabbitmq-server.service - RabbitMQ broker') && rabbitmqResult.stdout.includes('Active: active (running)')) {
      config.dependencies.rabbitMq.path = `${HOME_DIR}/credence/installation/rabbitmq`;
      config.dependencies.rabbitMq.isInstalled = true;
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else logger.error(`rabbitmq was not installed properly `);
  } catch (err) {
    throw new CliError(`Execution of rabbitmq.sh failed due to ${err.stack}`)
  }
}
async function rabbitmqRedhat9Install(directory) {
  try {
    const config = readConfig();
    logger.info(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/install/rabbitmq.sh ${directory} ${HOME_DIR}`);
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/install/rabbitmq.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of rabbitmq.sh failed due to ${err.stack}`)
  }
}
async function rabbitmqUbuntu20Install(directory) {
  try {

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/ubuntu20/scripts/install/rabbitmq.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of rabbitmq.sh failed due to ${err.stack}`)
  }
}

async function rabbitmqRedhat8_6Uninstall() {
  try {
    const config = readConfig();
    const rabbitmqResult = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat8_6/scripts/uninstall/rabbitmq.sh`)
    if (rabbitmqResult.stdout.length > 0) logger.info(`stdout : ${rabbitmqResult.stdout}`);
    if (rabbitmqResult.stderr.length > 0) logger.info(`stderr : ${rabbitmqResult.stderr}`);
    config.dependencies.rabbitMq.path = '';
    config.dependencies.rabbitMq.isInstalled = false;
    const newConfig = JSON.stringify(config, null, 2);
    fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
  } catch (err) {
    throw new CliError(`Execution of rabbitmq.sh failed due to ${err.stack}`)
  }
}
async function rabbitmqRedhat9Uninstall() {
  try {
    const value = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat9/scripts/uninstall/rabbitmq.sh ${config.init.path}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of rabbitqmq-.sh failed due to ${err.stack}`)
  }
}
async function rabbitmqUbuntu20Uninstall() {
  try {
    const value = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat8_6/scripts/uninstall/rabbitmq.sh ${config.init.path} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of rabbitqmq.sh failed due to ${err.stack}`)
  }
}

async function rabbitmqInstallation(directory) {
  try {
    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence/installation`))) {
      fs.mkdirSync(path.resolve(`${HOME_DIR}/credence/installation`));
    }
    const config = readConfig();

    switch (true) {
      case config.os.version.includes('8'):
        await rabbitmqRedhat8_6Install(directory);
        break;
      case config.os.version.includes('9'):
        await rabbitmqRedhat9Install(directory);
        break;
      case config.os.version.includes('ubuntu'):
        await rabbitmqUbuntu20Install(directory);
        break;
    }

  } catch (error) {
    logger.error(`Error in rabbitmqInstallation due to : ${error}`);
    throw new CliError(`Error in rabbitmqInstallation due to : ${error}`);
  }
}


async function rabbitMqUninstall(rabbitMqDetails) {
  try {
    if (rabbitMqDetails.isInstalled) {
      const config = readConfig();
      switch (true) {
        case config.os.version.includes('8'):
          await rabbitmqRedhat8_6Uninstall();
          break;
        case config.os.version.includes('9'):
          await rabbitmqRedhat9Uninstall();
          break;
        case config.os.version.includes('ubuntu'):
          await rabbitmqUbuntu20Uninstall();
          break;
      }
    } else {
      logger.info('rabbitmq is not installed');
    }
  } catch (error) {
    logger.error(`Error in rabbitMqUninstall due to : ${error}`);
    throw new CliError(`Error in rabbitMqUninstall due to : ${error}`);
  }
}

module.exports = { rabbitmqInstallation, rabbitMqUninstall };
