const extract = require('extract-zip');
const fs = require('fs');
const os = require('os');
const path = require('path');
const { readConfig } = require('../utils');

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');


async function wildflyInstallation(directory) {
  try {
    const config = readConfig();
    if (!config.services.applicationServer.isInstalled) {
      if (!fs.existsSync(path.resolve(`${directory}/Microservices/Application-Server`))) {
        logger.error('Provided path for installation files is incorrect');
      }
      if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence`))) {
        fs.mkdirSync(`${HOME_DIR}/credence`);
      }

      fs.copyFileSync(`${directory}/Microservices/Application-Server/wildfly.zip`, `${HOME_DIR}/credence/wildfly.zip`);
      await extract(`${HOME_DIR}/credence/wildfly.zip`, { dir: `${HOME_DIR}/credence` });
      fs.rmSync(path.resolve(`${HOME_DIR}/credence/wildfly.zip`), { force: true });

      if (fs.existsSync(path.resolve(`${HOME_DIR}/credence/wildfly`))) {
        config.services.applicationServer.path = path.resolve(`${HOME_DIR}/credence/wildfly/`);
        config.services.applicationServer.isInstalled = true;
        config.services.applicationServer.type = 'wildfly';
        const newConfig = JSON.stringify(config, null, 2);
        fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);

        fs.readFile(path.resolve(`${HOME_DIR}/credence/wildfly/modules/system/layers/base/org/quartz/main/funds_release.cfg`), (error, datam) => {
          if (error) {
            logger.error(`error while reading file : ${error}`);
          }

          let data = datam.toString();
          const newData = data
            .replace(/orgMsgLogPath=.*/, `orgMsgLogPath=${path.join(HOME_DIR, 'credence', 'wildfly', 'standalone', 'log')}`)
            .replace(/serverPath=.*/, `serverPath=${path.join(HOME_DIR, 'credence', 'wildfly', 'standalone', 'deployments', 'FUNDS_RELEASE.war')}`);
          fs.writeFile(`${HOME_DIR}/credence/wildfly/modules/system/layers/base/org/quartz/main/funds_release.cfg`, newData, (err) => {
            if (err) {
              logger.error(`error while writing file : ${err}`);
            }
          });
        });
      } else {
        logger.error('wildfly was not installed properly');
      }
    }
  } catch (error) {
    logger.error(`Error in wildflyInstallation due to : ${error}`);
    throw new CliError(`Error in wildflyInstallation due to : ${error}`);
  }
}

async function wildflyUninstall(wildflyDetails) {
  try {
    const config = readConfig();
    if (wildflyDetails.isInstalled === true) {
      fs.rmSync(wildflyDetails.path, { recursive: true, force: true })
      config.services.applicationServer.path = '';
      config.services.applicationServer.isInstalled = false;
      config.services.applicationServer.type = '';
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else {
      logger.info('wildfly is not installed');
    }
  } catch (error) {
    logger.error(`Error in wildflyUninstall due to : ${error}`);
    throw new CliError(`Error in wildflyUninstall due to : ${error}`);
  } finally {
    if (fs.existsSync(`${HOME_DIR}/credence/wildfly`)) fs.rmSync(`${HOME_DIR}/credence/wildfly`, { recursive: true, force: true })
  }
}

module.exports = { wildflyInstallation, wildflyUninstall };
